@@ -13,22 +13,12 @@ from pathlib import Path
 from typing import Optional, Union, Any, List
 from enum import Enum
 from abc import ABC, abstractmethod
-from openhcs.constants import Microscope
+from openhcs.constants import Microscope, VirtualComponents
 from openhcs.constants.constants import Backend
 
 # Import decorator for automatic decorator creation
 from openhcs.config_framework import auto_create_decorator
 
-# Import TilingLayout for TUI configuration
-try:
-    from textual_window import TilingLayout
-except ImportError:
-    # Fallback for when textual-window is not available
-    from enum import Enum
-    class TilingLayout(Enum):
-        FLOATING = "floating"
-        MASTER_DETAIL = "master_detail"
-
 logger = logging.getLogger(__name__)
 
 
@@ -84,6 +74,26 @@ class WellFilterMode(Enum):
     INCLUDE = "include"  # Materialize only specified wells
     EXCLUDE = "exclude"  # Materialize all wells except specified ones
 
+
+class NormalizationMethod(Enum):
+    """Normalization methods for experimental analysis."""
+    FOLD_CHANGE = "fold_change"  # value / control_mean
+    Z_SCORE = "z_score"  # (value - control_mean) / control_std
+    PERCENT_CONTROL = "percent_control"  # (value / control_mean) * 100
+
+
+class MicroscopeFormat(Enum):
+    """Supported microscope formats for experimental analysis."""
+    EDDU_CX5 = "EDDU_CX5"  # ThermoFisher CX5 format
+    EDDU_METAXPRESS = "EDDU_metaxpress"  # Molecular Devices MetaXpress format
+
+
+class TransportMode(Enum):
+    """ZMQ transport modes for local vs remote communication."""
+    IPC = "ipc"  # Inter-process communication (local only, no firewall prompts)
+    TCP = "tcp"  # Network sockets (supports remote, triggers firewall)
+
+
 @auto_create_decorator
 @dataclass(frozen=True)
 class GlobalPipelineConfig:
@@ -94,9 +104,7 @@ class GlobalPipelineConfig:
     num_workers: int = 1
     """Number of worker processes/threads for parallelizable tasks."""
 
-    test_str_field: str = "test"
-
-    materialization_results_path: Path = Path("results")
+    materialization_results_path: Path = field(default=Path("results"), metadata={'ui_hidden': True})
     """
     Path for materialized analysis results (CSV, JSON files from special outputs).
 
@@ -111,11 +119,11 @@ class GlobalPipelineConfig:
     by the sub_dir field in each step's step_materialization_config.
     """
 
-    microscope: Microscope = Microscope.AUTO
+    microscope: Microscope = field(default=Microscope.AUTO, metadata={'ui_hidden': True})
     """Default microscope type for auto-detection."""
 
     #use_threading: bool = field(default_factory=lambda: os.getenv('OPENHCS_USE_THREADING', 'false').lower() == 'true')
-    use_threading: bool = field(default_factory=lambda: os.getenv('OPENHCS_USE_THREADING', 'false').lower() == 'true')
+    use_threading: bool = field(default_factory=lambda: os.getenv('OPENHCS_USE_THREADING', 'false').lower() == 'true', metadata={'ui_hidden': True})
     """Use ThreadPoolExecutor instead of ProcessPoolExecutor for debugging. Reads from OPENHCS_USE_THREADING environment variable."""
 
     # Future extension point:
@@ -132,8 +140,13 @@ class GlobalPipelineConfig:
 from openhcs.utils.enum_factory import create_colormap_enum
 from openhcs.utils.display_config_factory import create_napari_display_config, create_fiji_display_config
 
-# Create the colormap enum using pure introspection
-NapariColormap = create_colormap_enum()
+
+# Import component order builder from factory module
+from openhcs.core.streaming_config_factory import build_component_order as _build_component_order
+
+# Create colormap enum with minimal set to avoid importing napari (→ dask → GPU libs)
+# The lazy=True parameter uses a hardcoded minimal set instead of introspecting napari
+NapariColormap = create_colormap_enum(lazy=True)
 
 
 class NapariDimensionMode(Enum):
@@ -149,35 +162,23 @@ class NapariVariableSizeHandling(Enum):
 
 
 # Create NapariDisplayConfig using factory
-from openhcs.constants import VirtualComponents
-
+# Note: Uses lazy colormap enum to avoid importing napari at module level
+# Note: component_order is automatically derived from VirtualComponents + AllComponents
+# This makes VirtualComponents the single source of truth
 NapariDisplayConfig = create_napari_display_config(
     colormap_enum=NapariColormap,
     dimension_mode_enum=NapariDimensionMode,
     variable_size_handling_enum=NapariVariableSizeHandling,
     virtual_components=VirtualComponents,
-    component_order=[
-        'step_name',
-        'step_index',
-        'source',
-        'well',
-        'site',
-        'channel',
-        'z_index',
-        'timepoint'
-    ],
+    component_order=_build_component_order(),  # Auto-generated from VirtualComponents
     virtual_component_defaults={
-        'step_name': NapariDimensionMode.SLICE,
-        'step_index': NapariDimensionMode.SLICE,
-        'source': NapariDimensionMode.SLICE
+        'source': NapariDimensionMode.SLICE  # Separate layers per step by default
     }
 )
 
 # Apply the global pipeline config decorator with ui_hidden=True
 # This config is only inherited by NapariStreamingConfig, so hide it from UI
 NapariDisplayConfig = global_pipeline_config(ui_hidden=True)(NapariDisplayConfig)
-# Mark the class directly as well for UI layer checks
-NapariDisplayConfig._ui_hidden = True
 
 
 # ============================================================================
@@ -214,23 +215,15 @@ class FijiDimensionMode(Enum):
 
 
 # Create FijiDisplayConfig using factory (with component-specific fields like Napari)
-# NOTE: step_name and step_index are EXCLUDED from component_order for Fiji
-# They are unreliable for ROIs (come from materialization context, not filename)
-# Only source and well are used for window grouping
+# Note: component_order is automatically derived from VirtualComponents + AllComponents
+# This makes VirtualComponents the single source of truth
 FijiDisplayConfig = create_fiji_display_config(
     lut_enum=FijiLUT,
     dimension_mode_enum=FijiDimensionMode,
     virtual_components=VirtualComponents,
-    component_order=[
-        'source',
-        'well',
-        'site',
-        'channel',
-        'z_index',
-        'timepoint'
-    ],
+    component_order=_build_component_order(),  # Auto-generated from VirtualComponents
     virtual_component_defaults={
-        # Only source is WINDOW for ROI->image mapping (well is already WINDOW in component_defaults)
+        # source is WINDOW by default for window grouping (well is already WINDOW in component_defaults)
         'source': FijiDimensionMode.WINDOW
     }
 )
@@ -335,9 +328,6 @@ class PlateMetadataConfig:
 @dataclass(frozen=True)
 class ExperimentalAnalysisConfig:
     """Configuration for experimental analysis system."""
-    enabled: bool = True
-    """Whether experimental analysis is enabled."""
-
     config_file_name: str = "config.xlsx"
     """Name of the experimental configuration Excel file."""
 
@@ -347,8 +337,8 @@ class ExperimentalAnalysisConfig:
     plate_groups_sheet_name: str = "plate_groups"
     """Name of the sheet containing plate group mappings."""
 
-    normalization_method: str = "fold_change"
-    """Normalization method: fold_change, z_score, percent_control."""
+    normalization_method: NormalizationMethod = NormalizationMethod.FOLD_CHANGE
+    """Normalization method for control-based normalization."""
 
     export_raw_results: bool = True
     """Whether to export raw (non-normalized) results."""
@@ -359,15 +349,9 @@ class ExperimentalAnalysisConfig:
     auto_detect_format: bool = True
     """Whether to automatically detect microscope format."""
 
-    default_format: Optional[str] = None
+    default_format: Optional[MicroscopeFormat] = None
     """Default format to use if auto-detection fails."""
 
-    enable_wells_exclusion: bool = True
-    """Whether to support wells exclusion from analysis (via 'Exclude Wells' row in config)."""
-
-    metaxpress_summary_enabled: bool = True
-    """Whether to generate MetaXpress-style summary output by default."""
-
 
 @global_pipeline_config
 @dataclass(frozen=True)
@@ -424,26 +408,9 @@ class StepMaterializationConfig(StepWellFilterConfig, PathPlanningConfig):
     sub_dir: str = "checkpoints"
     """Subdirectory for materialized outputs (different from global 'images')."""
 
+    enabled: bool = True
+    """Whether this materialization config is enabled. When False, config exists but materialization is disabled."""
 
-@global_pipeline_config
-@dataclass(frozen=True)
-class FunctionRegistryConfig:
-    """Configuration for function registry behavior across all libraries."""
-    enable_scalar_functions: bool = True
-    """
-    Whether to register functions that return scalars.
-    When True: Scalar-returning functions are wrapped as (array, scalar) tuples.
-    When False: Scalar-returning functions are filtered out entirely.
-    Applies uniformly to all libraries (CuPy, scikit-image, pyclesperanto).
-    """
-
-
-@global_pipeline_config
-@dataclass(frozen=True)
-class VisualizerConfig:
-    """Configuration for shared visualization system settings."""
-    temp_directory: Optional[Path] = None
-    """Directory for temporary visualization files. If None, will auto-create in system temp."""
 
 @global_pipeline_config
 @dataclass(frozen=True)
@@ -452,13 +419,27 @@ class StreamingDefaults:
     persistent: bool = True
     """Whether viewer stays open after pipeline completion."""
 
+    host: str = 'localhost'
+    """Host for streaming communication. Use 'localhost' for local, or remote IP for network streaming."""
+
+    port: int = None  # Subclasses must override with their specific default
+    """Port for streaming communication. Each streamer type has its own default."""
+
+    transport_mode: TransportMode = TransportMode.IPC
+    """ZMQ transport mode: IPC (local only, no firewall) or TCP (remote support, firewall prompts)."""
+
+    enabled: bool = True
+    """Whether this streaming config is enabled. When False, config exists but streaming is disabled."""
+
 @global_pipeline_config(ui_hidden=True)
 @dataclass(frozen=True)
 class StreamingConfig(StepWellFilterConfig, StreamingDefaults, ABC):
     """Abstract base configuration for streaming to visualizers.
 
     Uses multiple inheritance from StepWellFilterConfig and StreamingDefaults.
-    Inherited fields are automatically set to None by @global_pipeline_config(inherit_as_none=True).
+    Inherited fields (persistent, host, port, transport_mode) are automatically set to None
+    by @global_pipeline_config(inherit_as_none=True), enabling polymorphic access without
+    type-specific attribute names.
     """
 
     @property
@@ -467,6 +448,12 @@ class StreamingConfig(StepWellFilterConfig, StreamingDefaults, ABC):
         """Backend enum for this streaming type."""
         pass
 
+    @property
+    @abstractmethod
+    def viewer_type(self) -> str:
+        """Viewer type identifier (e.g., 'napari', 'fiji') for queue tracking and logging."""
+        pass
+
     @property
     @abstractmethod
     def step_plan_output_key(self) -> str:
@@ -484,160 +471,43 @@ class StreamingConfig(StepWellFilterConfig, StreamingDefaults, ABC):
         pass
 
 
-@global_pipeline_config
-@dataclass(frozen=True)
-class NapariStreamingConfig(StreamingConfig,NapariDisplayConfig):
-    """Configuration for napari streaming."""
-    napari_port: int = 5555
-    """Port for napari streaming communication."""
+# Auto-generate streaming configs using factory (reduces ~110 lines to ~20 lines)
+from openhcs.core.streaming_config_factory import create_streaming_config
 
-    napari_host: str = 'localhost'
-    """Host for napari streaming communication. Use 'localhost' for local, or remote IP for network streaming."""
-
-    @property
-    def backend(self) -> Backend:
-        return Backend.NAPARI_STREAM
-
-    @property
-    def step_plan_output_key(self) -> str:
-        return "napari_streaming_paths"
-
-    def get_streaming_kwargs(self, context) -> dict:
-        kwargs = {
-            "napari_port": self.napari_port,
-            "napari_host": self.napari_host,
-            "display_config": self  # self is now the display config
-        }
-
-        # Include microscope handler for component parsing
-        if context:
-            kwargs["microscope_handler"] = context.microscope_handler
-
-        return kwargs
-
-    def create_visualizer(self, filemanager, visualizer_config):
-        from openhcs.runtime.napari_stream_visualizer import NapariStreamVisualizer
-        return NapariStreamVisualizer(
-            filemanager,
-            visualizer_config,
-            viewer_title="OpenHCS Pipeline Visualization",
-            persistent=self.persistent,
-            napari_port=self.napari_port,
-            display_config=self  # self is now the display config
-        )
-
-
-@global_pipeline_config
-@dataclass(frozen=True)
-class FijiStreamingConfig(StreamingConfig, FijiDisplayConfig):
-    """
-    Configuration for Fiji streaming with display options.
-
-    Inherits from both StreamingConfig and FijiDisplayConfig to provide
-    feature parity with NapariStreamingConfig.
-    """
-    fiji_port: int = 5565  # Non-overlapping with Napari (5555-5564)
-    """Port for Fiji streaming communication (different default from Napari)."""
-
-    fiji_host: str = 'localhost'
-    """Host for Fiji streaming communication. Use 'localhost' for local, or remote IP for network streaming."""
-
-    fiji_executable_path: Optional[Path] = None
-    """Path to Fiji/ImageJ executable. If None, will auto-detect."""
-
-    @property
-    def backend(self) -> Backend:
-        return Backend.FIJI_STREAM
-
-    @property
-    def step_plan_output_key(self) -> str:
-        return "fiji_streaming_paths"
-
-    def get_streaming_kwargs(self, context) -> dict:
-        """Return kwargs needed for Fiji streaming backend (matches Napari pattern)."""
-        kwargs = {
-            "fiji_port": self.fiji_port,
-            "fiji_host": self.fiji_host,
-            "fiji_executable_path": self.fiji_executable_path,
-            "display_config": self  # self is now the display config
-        }
-
-        # Include microscope handler for component parsing
-        if context:
-            kwargs["microscope_handler"] = context.microscope_handler
-
-        return kwargs
-
-    def create_visualizer(self, filemanager, visualizer_config):
-        from openhcs.runtime.fiji_stream_visualizer import FijiStreamVisualizer
-        return FijiStreamVisualizer(
-            filemanager,
-            visualizer_config,
-            viewer_title="OpenHCS Fiji Visualization",
-            persistent=self.persistent,
-            fiji_port=self.fiji_port,
-            display_config=self
-        )
-
-@dataclass(frozen=True)
-class TilingKeybinding:
-    """Declarative mapping between key combination and window manager method."""
-    key: str
-    action: str  # method name that already exists
-    description: str
-
-@dataclass(frozen=True)
-class TilingKeybindings:
-    """Declarative mapping of tiling keybindings to existing methods."""
-
-    # Focus controls
-    focus_next: TilingKeybinding = TilingKeybinding("ctrl+j", "focus_next_window", "Focus Next Window")
-    focus_prev: TilingKeybinding = TilingKeybinding("ctrl+k", "focus_previous_window", "Focus Previous Window")
-
-    # Layout controls - map to wrapper methods
-    horizontal_split: TilingKeybinding = TilingKeybinding("ctrl+shift+h", "set_horizontal_split", "Horizontal Split")
-    vertical_split: TilingKeybinding = TilingKeybinding("ctrl+shift+v", "set_vertical_split", "Vertical Split")
-    grid_layout: TilingKeybinding = TilingKeybinding("ctrl+shift+g", "set_grid_layout", "Grid Layout")
-    master_detail: TilingKeybinding = TilingKeybinding("ctrl+shift+m", "set_master_detail", "Master Detail")
-    toggle_floating: TilingKeybinding = TilingKeybinding("ctrl+shift+f", "toggle_floating", "Toggle Floating")
-
-    # Window movement - map to extracted window_manager methods
-    move_window_prev: TilingKeybinding = TilingKeybinding("ctrl+shift+left", "move_focused_window_prev", "Move Window Left")
-    move_window_next: TilingKeybinding = TilingKeybinding("ctrl+shift+right", "move_focused_window_next", "Move Window Right")
-    rotate_left: TilingKeybinding = TilingKeybinding("ctrl+alt+left", "rotate_window_order_left", "Rotate Windows Left")
-    rotate_right: TilingKeybinding = TilingKeybinding("ctrl+alt+right", "rotate_window_order_right", "Rotate Windows Right")
-
-    # Gap controls
-    gap_increase: TilingKeybinding = TilingKeybinding("ctrl+plus", "gap_increase", "Increase Gap")
-    gap_decrease: TilingKeybinding = TilingKeybinding("ctrl+minus", "gap_decrease", "Decrease Gap")
-
-    # Bulk operations
-    minimize_all: TilingKeybinding = TilingKeybinding("ctrl+shift+d", "minimize_all_windows", "Minimize All")
-    open_all: TilingKeybinding = TilingKeybinding("ctrl+shift+o", "open_all_windows", "Open All")
-
-
-
-@dataclass(frozen=True)
-class TUIConfig:
-    """Configuration for OpenHCS Textual User Interface."""
-    default_tiling_layout: TilingLayout = TilingLayout.MASTER_DETAIL
-    """Default tiling layout for window manager on startup."""
-
-    default_window_gap: int = 1
-    """Default gap between windows in tiling mode (in characters)."""
-
-    enable_startup_notification: bool = True
-    """Whether to show notification about tiling mode on startup."""
-
-    keybindings: TilingKeybindings = field(default_factory=TilingKeybindings)
-    """Declarative mapping of all tiling keybindings."""
+NapariStreamingConfig = create_streaming_config(
+    viewer_name='napari',
+    port=5555,
+    backend=Backend.NAPARI_STREAM,
+    display_config_class=NapariDisplayConfig,
+    visualizer_module='openhcs.runtime.napari_stream_visualizer',
+    visualizer_class_name='NapariStreamVisualizer'
+)
 
+FijiStreamingConfig = create_streaming_config(
+    viewer_name='fiji',
+    port=5565,
+    backend=Backend.FIJI_STREAM,
+    display_config_class=FijiDisplayConfig,
+    visualizer_module='openhcs.runtime.fiji_stream_visualizer',
+    visualizer_class_name='FijiStreamVisualizer',
+    extra_fields={
+        'fiji_executable_path': (Optional[Path], None)
+    }
+)
 
 # Inject all accumulated fields at the end of module loading
 from openhcs.config_framework.lazy_factory import _inject_all_pending_fields
 _inject_all_pending_fields()
 
 
+# ============================================================================
+# Streaming Port Utilities
+# ============================================================================
+
+# Import streaming port utility from factory module
+from openhcs.core.streaming_config_factory import get_all_streaming_ports
+
+
 # ============================================================================
 # Configuration Framework Initialization
 # ============================================================================
@@ -653,15 +523,8 @@ set_base_config_type(GlobalPipelineConfig)
 
 logger.debug("Configuration framework initialized with OpenHCS types")
 
-# PERFORMANCE OPTIMIZATION: Pre-warm analysis caches at import time
-# This eliminates the 1000ms+ first-load penalty when opening config windows
-try:
-    from openhcs.config_framework import prewarm_config_analysis_cache
-    # Warm config hierarchy cache (for config windows)
-    prewarm_config_analysis_cache(GlobalPipelineConfig)
-except ImportError:
-    # Circular import during subprocess initialization - cache warming not needed
-    # for non-UI execution contexts (ZMQ server, workers, etc.)
-    logger.debug("Skipping config cache warming (circular import during subprocess init)")
+# PERFORMANCE OPTIMIZATION: Cache warming is now done asynchronously in GUI startup
+# to avoid blocking imports. For non-GUI contexts (CLI, subprocess), cache warming
+# happens on-demand when config windows are first opened.
 
 # NOTE: Step editor cache warming is done in openhcs.core.steps.__init__ to avoid circular imports
