@@ -40,14 +40,19 @@ from openhcs.config_framework.lazy_factory import resolve_lazy_configurations_fo
 from openhcs.microscopes import create_microscope_handler
 from openhcs.microscopes.microscope_base import MicroscopeHandler
 
-# Conditional analysis import - skip in subprocess runner mode
-if os.getenv('OPENHCS_SUBPROCESS_NO_GPU') == '1':
-    # Subprocess runner mode - create placeholder
-    def consolidate_analysis_results(*args, **kwargs):
-        """Placeholder for subprocess runner mode."""
-        raise RuntimeError("Analysis consolidation not available in subprocess runner mode")
-else:
-    from openhcs.processing.backends.analysis.consolidate_analysis_results import consolidate_analysis_results
+# Lazy import of consolidate_analysis_results to avoid blocking GUI startup
+# This function imports GPU libraries, so we defer it until first use
+def _get_consolidate_analysis_results():
+    """Lazy import of consolidate_analysis_results function."""
+    if os.getenv('OPENHCS_SUBPROCESS_NO_GPU') == '1':
+        # Subprocess runner mode - create placeholder
+        def consolidate_analysis_results(*args, **kwargs):
+            """Placeholder for subprocess runner mode."""
+            raise RuntimeError("Analysis consolidation not available in subprocess runner mode")
+        return consolidate_analysis_results
+    else:
+        from openhcs.processing.backends.analysis.consolidate_analysis_results import consolidate_analysis_results
+        return consolidate_analysis_results
 
 # Import generic component system - required for orchestrator functionality
 
@@ -62,9 +67,8 @@ except ImportError:
 
 # Optional GPU memory management imports
 try:
-    from openhcs.core.memory.gpu_cleanup import log_gpu_memory_usage, cleanup_all_gpu_frameworks
+    from openhcs.core.memory.gpu_cleanup import cleanup_all_gpu_frameworks
 except ImportError:
-    log_gpu_memory_usage = None
     cleanup_all_gpu_frameworks = None
 
 
@@ -113,18 +117,8 @@ def _create_merged_config(pipeline_config: 'PipelineConfig', global_config: Glob
         else:
             global_value = getattr(global_config, field.name)
             merged_config_values[field.name] = global_value
-            if field.name == 'step_well_filter_config':
-                print(f"🔍 MERGE DEBUG: Using global_config value: {global_value}")
 
     result = GlobalPipelineConfig(**merged_config_values)
-
-    # DEBUG: Check what the result looks like
-    if hasattr(result, 'step_well_filter_config'):
-        step_config = getattr(result, 'step_well_filter_config')
-        if hasattr(step_config, 'well_filter'):
-            well_filter_value = getattr(step_config, 'well_filter')
-            print(f"🔍 MERGE DEBUG: Final result has step_well_filter_config.well_filter = {well_filter_value}")
-
     return result
 
 
@@ -145,16 +139,15 @@ def _execute_single_axis_static(
         visualizer: Optional Napari visualizer (not used in multiprocessing)
     """
     axis_id = frozen_context.axis_id
-    logger.info(f"🔥 SINGLE_AXIS: Starting execution for axis {axis_id}")
 
     # NUCLEAR VALIDATION
     if not frozen_context.is_frozen():
-        error_msg = f"🔥 SINGLE_AXIS ERROR: Context for axis {axis_id} is not frozen before execution"
+        error_msg = f"Context for axis {axis_id} is not frozen before execution"
         logger.error(error_msg)
         raise RuntimeError(error_msg)
 
     if not pipeline_definition:
-        error_msg = f"🔥 SINGLE_AXIS ERROR: Empty pipeline_definition for axis {axis_id}"
+        error_msg = f"Empty pipeline_definition for axis {axis_id}"
         logger.error(error_msg)
         raise RuntimeError(error_msg)
 
@@ -162,17 +155,14 @@ def _execute_single_axis_static(
     for step_index, step in enumerate(pipeline_definition):
         step_name = frozen_context.step_plans[step_index]["step_name"]
 
-        logger.info(f"🔥 SINGLE_AXIS: Executing step {step_index+1}/{len(pipeline_definition)} - {step_name} for axis {axis_id}")
-
         # Verify step has process method (should always be true for AbstractStep subclasses)
         if not hasattr(step, 'process'):
-            error_msg = f"🔥 SINGLE_AXIS ERROR: Step {step_index+1} missing process method for axis {axis_id}"
+            error_msg = f"Step {step_index+1} missing process method for axis {axis_id}"
             logger.error(error_msg)
             raise RuntimeError(error_msg)
 
         # Call process method on step instance
         step.process(frozen_context, step_index)
-        logger.info(f"🔥 SINGLE_AXIS: Step {step_index+1}/{len(pipeline_definition)} - {step_name} completed for axis {axis_id}")
 
         # Handle visualization if requested
         if visualizer:
@@ -239,11 +229,6 @@ def _configure_worker_logging(log_file_base: str):
 
     # Get worker logger
     worker_logger = logging.getLogger("openhcs.worker")
-    worker_logger.info(f"🔥 WORKER: Process {worker_pid} (ID: {worker_id}) logging configured")
-    worker_logger.info(f"🔥 WORKER: All logs writing to: {worker_log_file}")
-
-    # Log import hook installation status
-    worker_logger.info("🔥 WORKER: Import hook installed for auto-discovered functions")
 
 
 def _configure_worker_with_gpu(log_file_base: str, global_config_dict: dict):
@@ -273,12 +258,9 @@ def _configure_worker_with_gpu(log_file_base: str, global_config_dict: dict):
         # Set up basic logging for worker messages
         logging.basicConfig(level=logging.INFO)
         worker_logger = logging.getLogger("openhcs.worker")
-        worker_logger.info("🔥 WORKER: No log file base provided, using basic logging")
 
     # Initialize function registry for this worker process
     try:
-        worker_logger.info("🔥 WORKER: Initializing function registry for worker process")
-
         # Import and initialize function registry (will auto-discover all libraries)
         import openhcs.processing.func_registry as func_registry_module
 
@@ -287,17 +269,12 @@ def _configure_worker_with_gpu(log_file_base: str, global_config_dict: dict):
             if not func_registry_module._registry_initialized:
                 func_registry_module._auto_initialize_registry()
 
-        worker_logger.info("🔥 WORKER: Function registry initialized successfully")
-
     except Exception as e:
-        worker_logger.error(f"🔥 WORKER: Failed to initialize function registry: {e}")
         # Don't raise - let worker continue, registry will auto-init on first function call
-        worker_logger.warning("🔥 WORKER: Function registry will auto-initialize on first function call")
+        pass
 
     # Initialize GPU registry for this worker process
     try:
-        worker_logger.info("🔥 WORKER: Initializing GPU registry for worker process")
-
         # Reconstruct global config from dict
         from openhcs.core.config import GlobalPipelineConfig
         global_config = GlobalPipelineConfig(**global_config_dict)
@@ -306,12 +283,9 @@ def _configure_worker_with_gpu(log_file_base: str, global_config_dict: dict):
         from openhcs.core.orchestrator.gpu_scheduler import setup_global_gpu_registry
         setup_global_gpu_registry(global_config)
 
-        worker_logger.info("🔥 WORKER: GPU registry initialized successfully")
-
     except Exception as e:
-        worker_logger.error(f"🔥 WORKER: Failed to initialize GPU registry: {e}")
         # Don't raise - let worker continue without GPU if needed
-        worker_logger.warning("🔥 WORKER: Continuing without GPU registry - GPU functions may fail")
+        pass
 
 
 # Global variable to store log file base for worker processes
@@ -419,7 +393,9 @@ class PipelineOrchestrator(ContextProvider):
             logger.info("PipelineOrchestrator using provided StorageRegistry instance.")
         else:
             # Create a copy of the global registry to avoid modifying shared state
-            from openhcs.io.base import storage_registry as global_storage_registry
+            from openhcs.io.base import storage_registry as global_storage_registry, ensure_storage_registry
+            # Ensure registry is initialized before copying
+            ensure_storage_registry()
             self.registry = global_storage_registry.copy()
             logger.info("PipelineOrchestrator created its own StorageRegistry instance (copy of global).")
 
@@ -484,60 +460,57 @@ class PipelineOrchestrator(ContextProvider):
         duplicating viewer instances. Viewers are tracked by (backend_name, port) key.
 
         Args:
-            config: Streaming config (NapariStreamingConfig or FijiStreamingConfig)
+            config: Streaming config (any StreamingConfig subclass)
             vis_config: Optional visualizer config (can be None for image browser)
 
         Returns:
-            Visualizer instance (NapariStreamVisualizer or FijiStreamVisualizer)
+            Visualizer instance
         """
-        from openhcs.core.config import NapariStreamingConfig, FijiStreamingConfig
+        from openhcs.core.config import StreamingConfig
 
-        # Determine key based on config type (use isinstance for type checking)
-        if isinstance(config, NapariStreamingConfig):
-            key = ('napari', config.napari_port)
-            port = config.napari_port
-        elif isinstance(config, FijiStreamingConfig):
-            key = ('fiji', config.fiji_port)
-            port = config.fiji_port
+        # Generic streaming config handling using polymorphic attributes
+        if isinstance(config, StreamingConfig):
+            # Start global ack listener (must be before viewers connect)
+            from openhcs.runtime.zmq_base import start_global_ack_listener
+            start_global_ack_listener(config.transport_mode)
+
+            # Pre-create queue tracker using polymorphic attributes
+            from openhcs.runtime.queue_tracker import GlobalQueueTrackerRegistry
+            registry = GlobalQueueTrackerRegistry()
+            registry.get_or_create_tracker(config.port, config.viewer_type)
+            logger.info(f"🔬 ORCHESTRATOR: Pre-created queue tracker for {config.viewer_type} on port {config.port}")
+
+            key = (config.viewer_type, config.port)
         else:
             backend_name = config.backend.name if hasattr(config, 'backend') else 'unknown'
             key = (backend_name,)
-            port = None
 
         # Check if we already have a visualizer for this key
         if key in self._visualizers:
             vis = self._visualizers[key]
-            # Check if it's still running
             if vis.is_running:
-                logger.info(f"🔬 ORCHESTRATOR: Reusing existing visualizer for {key}")
                 return vis
             else:
-                logger.info(f"🔬 ORCHESTRATOR: Existing visualizer for {key} is not running, creating new one")
                 del self._visualizers[key]
 
-        # Create new visualizer
-        logger.info(f"🔬 ORCHESTRATOR: Creating new visualizer for {key} (persistent={config.persistent})")
+        # Create new visualizer using polymorphic create_visualizer method
         vis = config.create_visualizer(self.filemanager, vis_config)
 
-        # Start viewer asynchronously for both Napari and Fiji
-        if isinstance(config, (NapariStreamingConfig, FijiStreamingConfig)):
-            logger.info(f"🔬 ORCHESTRATOR: Starting {key[0]} visualizer asynchronously on port {port}")
+        # Start viewer asynchronously for streaming configs
+        if isinstance(config, StreamingConfig):
             vis.start_viewer(async_mode=True)
 
-            # Ping the server to set it to ready state (required for Fiji to process data messages)
-            # Do this in a background thread to avoid blocking
+            # Ping server to set ready state (background thread to avoid blocking)
             import threading
             def ping_server():
                 import time
                 time.sleep(1.0)  # Give server time to start
                 if hasattr(vis, '_wait_for_server_ready'):
                     vis._wait_for_server_ready(timeout=10.0)
-                    logger.info(f"🔬 ORCHESTRATOR: {key[0]} visualizer on port {port} is ready")
 
             thread = threading.Thread(target=ping_server, daemon=True)
             thread.start()
         else:
-            logger.info(f"🔬 ORCHESTRATOR: Starting visualizer: {vis}")
             vis.start_viewer()
 
         # Store in cache
@@ -575,17 +548,19 @@ class PipelineOrchestrator(ContextProvider):
         Must be called before other processing methods.
         Returns self for chaining.
         """
+        logger.info(f"🔥 INIT: initialize() called for plate: {self.plate_path}")
         if self._initialized:
             logger.info("Orchestrator already initialized.")
             return self
 
         try:
+            logger.info(f"🔥 INIT: About to call initialize_microscope_handler()")
             self.initialize_microscope_handler()
 
             # Delegate workspace initialization to microscope handler
             logger.info("Initializing workspace with microscope handler...")
             actual_image_dir = self.microscope_handler.initialize_workspace(
-                self.plate_path, workspace_path, self.filemanager
+                self.plate_path, self.filemanager
             )
 
             # Use the actual image directory returned by the microscope handler
@@ -614,6 +589,9 @@ class PipelineOrchestrator(ContextProvider):
                 self._component_keys_cache
             )
 
+            # Ensure complete OpenHCS metadata exists
+            self._ensure_openhcs_metadata()
+
             logger.info("PipelineOrchestrator fully initialized with cached component keys and metadata.")
             return self
         except Exception as e:
@@ -624,6 +602,43 @@ class PipelineOrchestrator(ContextProvider):
     def is_initialized(self) -> bool:
         return self._initialized
 
+    def _ensure_openhcs_metadata(self) -> None:
+        """Ensure complete OpenHCS metadata exists for the plate.
+
+        Uses the same context creation logic as pipeline execution to get full metadata
+        with channel names from metadata files (HTD, Index.xml, etc).
+
+        Skips OMERO and other non-disk-based microscope handlers since they don't have
+        real disk directories.
+        """
+        from openhcs.microscopes.openhcs import OpenHCSMetadataGenerator
+
+        # Skip metadata creation for OMERO and other non-disk-based handlers
+        # OMERO uses virtual paths like /omero/plate_1 which are not real directories
+        if self.microscope_handler.microscope_type == 'omero':
+            logger.debug("Skipping metadata creation for OMERO plate (uses virtual paths)")
+            return
+
+        # For plates with virtual workspace, metadata is already created by _build_virtual_mapping()
+        # We just need to add the component metadata to the existing "." subdirectory
+        from openhcs.io.metadata_writer import get_subdirectory_name
+        subdir_name = get_subdirectory_name(self.input_dir, self.plate_path)
+
+        # Create context using SAME logic as create_context() to get full metadata
+        context = self.create_context(axis_id="metadata_init")
+
+        # Create metadata (will skip if already complete)
+        generator = OpenHCSMetadataGenerator(self.filemanager)
+        generator.create_metadata(
+            context,
+            str(self.input_dir),
+            "disk",
+            is_main=True,
+            plate_root=str(self.plate_path),
+            sub_dir=subdir_name,
+            skip_if_complete=True
+        )
+
     def get_results_path(self) -> Path:
         """Get the results directory path for this orchestrator's plate.
 
@@ -867,73 +882,51 @@ class PipelineOrchestrator(ContextProvider):
             actual_max_workers = 1
 
         # 🔬 AUTOMATIC VISUALIZER CREATION: Create visualizers if compiler detected streaming
-        # Support multiple napari instances based on unique ports
         visualizers = []
         if visualizer is None:
-            # Collect all unique napari ports across all compiled contexts
-            unique_napari_ports = set()
+            from openhcs.core.config import StreamingConfig
+
+            # Collect unique configs (deduplicate by viewer_type + port)
+            unique_configs = {}
             for ctx in compiled_contexts.values():
                 for visualizer_info in ctx.required_visualizers:
                     config = visualizer_info['config']
-                    # Check if this is a napari streaming config with a port
-                    if hasattr(config, 'napari_port'):
-                        unique_napari_ports.add(config.napari_port)
+                    key = (config.viewer_type, config.port) if isinstance(config, StreamingConfig) else (config.backend.name,)
+                    if key not in unique_configs:
+                        unique_configs[key] = (config, ctx.visualizer_config)
 
-            logger.info(f"🔬 ORCHESTRATOR: Found {len(unique_napari_ports)} unique napari ports: {unique_napari_ports}")
+            # Create visualizers
+            for config, vis_config in unique_configs.values():
+                visualizers.append(self.get_or_create_visualizer(config, vis_config))
 
-            # Collect all unique visualizer configs across all contexts
-            # Key: (backend_name, port) for napari, (backend_name,) for others
-            # Value: config object
-            unique_visualizer_configs = {}
-
-            for ctx in compiled_contexts.values():
-                for visualizer_info in ctx.required_visualizers:
-                    config = visualizer_info['config']
-
-                    # For napari configs, use port as part of the key
-                    if hasattr(config, 'napari_port'):
-                        key = ('napari', config.napari_port)
-                        # Store the first config we see for each port
-                        # All configs for the same port should be identical after resolution
-                        if key not in unique_visualizer_configs:
-                            logger.info(f"🔬 ORCHESTRATOR: Found first config for port {config.napari_port}: persistent={config.persistent}")
-                            unique_visualizer_configs[key] = (config, ctx.visualizer_config)
-                        else:
-                            # Log if we see a different config for the same port
-                            existing_config = unique_visualizer_configs[key][0]
-                            if existing_config.persistent != config.persistent:
-                                logger.warning(f"🔬 ORCHESTRATOR: Conflicting persistent values for port {config.napari_port}: existing={existing_config.persistent}, new={config.persistent}")
-                    else:
-                        # For non-napari visualizers, use backend name as key
-                        backend_name = config.backend.name if hasattr(config, 'backend') else 'unknown'
-                        key = (backend_name,)
-                        if key not in unique_visualizer_configs:
-                            unique_visualizer_configs[key] = (config, ctx.visualizer_config)
-
-            # Create and start all visualizers using shared infrastructure
-            for key, (config, vis_config) in unique_visualizer_configs.items():
-                vis = self.get_or_create_visualizer(config, vis_config)
-                visualizers.append(vis)
-
-            # Wait for all napari viewers to be ready before starting pipeline
+            # Wait for all streaming viewers to be ready before starting pipeline
             # This ensures viewers are available to receive images
-            napari_visualizers = [v for v in visualizers if hasattr(v, 'napari_port')]
-            if napari_visualizers:
-                logger.info(f"🔬 ORCHESTRATOR: Waiting for {len(napari_visualizers)} napari viewer(s) to be ready...")
+            if visualizers:
+                logger.info(f"🔬 ORCHESTRATOR: Waiting for {len(visualizers)} streaming viewer(s) to be ready...")
                 import time
                 max_wait = 30.0  # Maximum wait time in seconds
                 start_time = time.time()
 
                 while time.time() - start_time < max_wait:
-                    all_ready = all(v.is_running for v in napari_visualizers)
+                    all_ready = all(v.is_running for v in visualizers)
                     if all_ready:
-                        logger.info("🔬 ORCHESTRATOR: All napari viewers are ready!")
+                        logger.info("🔬 ORCHESTRATOR: All streaming viewers are ready!")
                         break
                     time.sleep(0.2)  # Check every 200ms
                 else:
-                    # Timeout - log which viewers aren't ready
-                    not_ready = [v.napari_port for v in napari_visualizers if not v.is_running]
-                    logger.warning(f"🔬 ORCHESTRATOR: Timeout waiting for napari viewers. Not ready: {not_ready}")
+                    # Timeout - log which viewers aren't ready (use generic port attribute)
+                    not_ready = [v.port for v in visualizers if not v.is_running]
+                    logger.warning(f"🔬 ORCHESTRATOR: Timeout waiting for streaming viewers. Not ready: {not_ready}")
+
+                # Clear viewer state for new pipeline run to prevent accumulation
+                logger.info("🔬 ORCHESTRATOR: Clearing streaming viewer state for new pipeline run...")
+                for vis in visualizers:
+                    if hasattr(vis, 'clear_viewer_state'):
+                        success = vis.clear_viewer_state()
+                        if success:
+                            logger.info(f"🔬 ORCHESTRATOR: Cleared state for viewer on port {vis.port}")
+                        else:
+                            logger.warning(f"🔬 ORCHESTRATOR: Failed to clear state for viewer on port {vis.port}")
 
             # For backwards compatibility, set visualizer to the first one
             visualizer = visualizers[0] if visualizers else None
@@ -941,13 +934,6 @@ class PipelineOrchestrator(ContextProvider):
         self._state = OrchestratorState.EXECUTING
         logger.info(f"Starting execution for {len(compiled_contexts)} axis values with max_workers={actual_max_workers}.")
 
-        # 🔍 VRAM TRACKING: Log initial memory state
-        try:
-            if log_gpu_memory_usage:
-                log_gpu_memory_usage("plate execution start")
-        except Exception:
-            pass
-
         try:
             execution_results: Dict[str, Dict[str, Any]] = {}
 
@@ -1142,7 +1128,8 @@ class PipelineOrchestrator(ContextProvider):
                             logger.info(f"🔄 CONSOLIDATION: Using well IDs: {axis_ids}")
 
                             logger.info("🔥 ORCHESTRATOR: Calling consolidate_analysis_results()...")
-                            consolidate_analysis_results(
+                            consolidate_fn = _get_consolidate_analysis_results()
+                            consolidate_fn(
                                 results_directory=str(results_dir),
                                 well_ids=axis_ids,
                                 consolidation_config=shared_context.analysis_consolidation_config,
@@ -1276,7 +1263,7 @@ class PipelineOrchestrator(ContextProvider):
         # Single pass through all filenames - extract all components at once
         try:
             # Use primary backend from microscope handler
-            backend_to_use = self.microscope_handler.get_primary_backend(self.input_dir)
+            backend_to_use = self.microscope_handler.get_primary_backend(self.input_dir, self.filemanager)
             logger.debug(f"Using backend '{backend_to_use}' for file listing based on available backends")
 
             filenames = self.filemanager.list_files(str(self.input_dir), backend_to_use, extensions=DEFAULT_IMAGE_EXTENSIONS)
