@@ -85,7 +85,10 @@ def _refresh_function_objects_in_steps(pipeline_definition: List[AbstractStep])
 
 
 def _refresh_function_object(func_value):
-    """Convert function objects to picklable FunctionReference objects."""
+    """Convert function objects to picklable FunctionReference objects.
+
+    Also filters out functions with enabled=False at compile time.
+    """
     try:
         if callable(func_value) and hasattr(func_value, '__module__'):
             # Single function → FunctionReference
@@ -94,17 +97,31 @@ def _refresh_function_object(func_value):
         elif isinstance(func_value, tuple) and len(func_value) == 2:
             # Function with parameters tuple → (FunctionReference, params)
             func, params = func_value
+
+            # Check if function is disabled via enabled parameter
+            if isinstance(params, dict) and params.get('enabled', True) is False:
+                import logging
+                logger = logging.getLogger(__name__)
+                func_name = getattr(func, '__name__', str(func))
+                logger.info(f"🔧 COMPILE-TIME FILTER: Removing disabled function '{func_name}' from pipeline")
+                return None  # Mark for removal
+
             if callable(func):
                 func_ref = _refresh_function_object(func)
+                # Remove 'enabled' from params since it's not a real function parameter
+                if isinstance(params, dict) and 'enabled' in params:
+                    params = {k: v for k, v in params.items() if k != 'enabled'}
                 return (func_ref, params)
 
         elif isinstance(func_value, list):
-            # List of functions → List of FunctionReferences
-            return [_refresh_function_object(item) for item in func_value]
+            # List of functions → List of FunctionReferences (filter out None)
+            refreshed = [_refresh_function_object(item) for item in func_value]
+            return [item for item in refreshed if item is not None]
 
         elif isinstance(func_value, dict):
-            # Dict of functions → Dict of FunctionReferences
-            return {key: _refresh_function_object(value) for key, value in func_value.items()}
+            # Dict of functions → Dict of FunctionReferences (filter out None values)
+            refreshed = {key: _refresh_function_object(value) for key, value in func_value.items()}
+            return {key: value for key, value in refreshed.items() if value is not None}
 
     except Exception as e:
         import logging
@@ -211,18 +228,13 @@ class PipelineCompiler:
             context.step_plans = {} # Ensure step_plans dict exists
 
         # === VISUALIZER CONFIG EXTRACTION ===
-        # Extract visualizer config from orchestrator.pipeline_config
-        # The caller has already set up config_context(orchestrator.pipeline_config)
-        # so we can just access the field directly - lazy resolution happens automatically
-        context.visualizer_config = orchestrator.pipeline_config.visualizer_config
-
-        # === BACKWARDS COMPATIBILITY PREPROCESSING ===
-        # Ensure all steps have complete attribute sets based on AbstractStep constructor
-        # This must happen before any other compilation logic to eliminate defensive programming
-        logger.debug("🔧 BACKWARDS COMPATIBILITY: Normalizing step attributes...")
-        _normalize_step_attributes(steps_definition)
-
+        # visualizer_config is a legacy parameter that's passed to visualizers but never used
+        # The actual display configuration comes from the display_config parameter
+        # Set to None for backward compatibility with orchestrator code
+        context.visualizer_config = None
 
+        # Note: _normalize_step_attributes is now called in compile_pipelines() before filtering
+        # to ensure old pickled steps have the 'enabled' attribute before we check it
 
         # Pre-initialize step_plans with basic entries for each step
         # Use step index as key instead of step_id for multiprocessing compatibility
@@ -252,14 +264,25 @@ class PipelineCompiler:
                 already_zarr = Backend.ZARR in available_backends
 
                 if not already_zarr:
-                    # Inject input conversion config using existing PathPlanningConfig pattern
-                    path_config = orchestrator.pipeline_config.path_planning_config
-                    conversion_config = PathPlanningConfig(
-                        output_dir_suffix=".zarr",  # Zarr store suffix
-                        global_output_folder=plate_path.parent,  # Parent of plate
-                        sub_dir=path_config.sub_dir  # Use same sub_dir (e.g., "images")
-                    )
-                    context.step_plans[0]["input_conversion_config"] = conversion_config
+                    # Determine if input uses virtual workspace
+                    from openhcs.microscopes.openhcs import OpenHCSMetadataHandler
+                    from openhcs.io.metadata_writer import get_subdirectory_name
+
+                    openhcs_metadata_handler = OpenHCSMetadataHandler(context.filemanager)
+                    metadata = openhcs_metadata_handler._load_metadata_dict(plate_path)
+                    subdirs = metadata["subdirectories"]
+
+                    # Get actual subdirectory from input_dir
+                    original_subdir = get_subdirectory_name(context.input_dir, plate_path)
+                    uses_virtual_workspace = Backend.VIRTUAL_WORKSPACE.value in subdirs[original_subdir]["available_backends"]
+
+                    zarr_subdir = "zarr" if uses_virtual_workspace else original_subdir
+                    conversion_dir = plate_path / zarr_subdir
+
+                    context.step_plans[0]["input_conversion_dir"] = str(conversion_dir)
+                    context.step_plans[0]["input_conversion_backend"] = MaterializationBackend.ZARR.value
+                    context.step_plans[0]["input_conversion_uses_virtual_workspace"] = uses_virtual_workspace
+                    context.step_plans[0]["input_conversion_original_subdir"] = original_subdir
                     logger.debug(f"Input conversion to zarr enabled for first step: {first_step.name}")
 
         # The axis_id and base_input_dir are available from the context object.
@@ -366,73 +389,73 @@ class PipelineCompiler:
                     # Configs are already resolved to base configs at line 277
                     # No need to call to_base_config() again - that's legacy code
 
-                    # Unified handling: compute inclusion for any WellFilterConfig (StreamingConfig subclasses it)
-                    is_streaming = config is not None and isinstance(config, StreamingConfig)
-                    is_wellfilter = config is not None and isinstance(config, WellFilterConfig)
+                    # Skip None configs
+                    if config is None:
+                        continue
 
-                    include_config = False
-                    if is_wellfilter:
-                        # Check if this config has a well filter and if current axis matches
-                        should_include_config = True
-                        if config.well_filter is not None:
-                            config_filter = step_axis_filters.get(attr_name)
-                            if config_filter:
-                                # Apply axis filter logic
-                                axis_in_filter = context.axis_id in config_filter['resolved_axis_values']
-                                should_include_config = (
-                                    axis_in_filter if config_filter['filter_mode'] == WellFilterMode.INCLUDE
-                                    else not axis_in_filter
-                                )
-                        if should_include_config:
-                            include_config = True
+                    # CRITICAL: Check enabled field first (fail-fast for disabled configs)
+                    if hasattr(config, 'enabled') and not config.enabled:
+                        continue
 
-                    # Only add the config to the plan if it's included (or not a WellFilterConfig)
-                    if include_config or (not is_wellfilter and config is not None):
+                    # Check well filter matching (only for WellFilterConfig instances)
+                    include_config = True
+                    if isinstance(config, WellFilterConfig) and config.well_filter is not None:
+                        config_filter = step_axis_filters.get(attr_name)
+                        if config_filter:
+                            # Apply axis filter logic
+                            axis_in_filter = context.axis_id in config_filter['resolved_axis_values']
+                            include_config = (
+                                axis_in_filter if config_filter['filter_mode'] == WellFilterMode.INCLUDE
+                                else not axis_in_filter
+                            )
+
+                    # Add config to plan if it passed all checks
+                    if include_config:
                         current_plan[attr_name] = config
 
-                    # Streaming extras only apply if the streaming config is included
-                    if is_streaming and include_config:
-                        # Validate that the visualizer can actually be created
-                        try:
-                            # Only validate configs that actually have a backend (real streaming configs)
-                            if not hasattr(config, 'backend'):
-                                continue
+                        # Add streaming extras if this is a streaming config
+                        if isinstance(config, StreamingConfig):
+                            # Validate that the visualizer can actually be created
+                            try:
+                                # Only validate configs that actually have a backend (real streaming configs)
+                                if not hasattr(config, 'backend'):
+                                    continue
 
-                            # Test visualizer creation without actually creating it
-                            if hasattr(config, 'create_visualizer'):
-                                # For napari, check if napari is available and environment supports GUI
-                                if config.backend.name == 'NAPARI_STREAM':
-                                    from openhcs.utils.import_utils import optional_import
-                                    import os
+                                # Test visualizer creation without actually creating it
+                                if hasattr(config, 'create_visualizer'):
+                                    # For napari, check if napari is available and environment supports GUI
+                                    if config.backend.name == 'NAPARI_STREAM':
+                                        from openhcs.utils.import_utils import optional_import
+                                        import os
 
-                                    # Check if running in headless/CI environment
-                                    # CPU-only mode does NOT imply headless - you can run CPU mode with napari
-                                    is_headless = (
-                                        os.getenv('CI', 'false').lower() == 'true' or
-                                        os.getenv('OPENHCS_HEADLESS', 'false').lower() == 'true' or
-                                        os.getenv('DISPLAY') is None
-                                    )
+                                        # Check if running in headless/CI environment
+                                        # CPU-only mode does NOT imply headless - you can run CPU mode with napari
+                                        is_headless = (
+                                            os.getenv('CI', 'false').lower() == 'true' or
+                                            os.getenv('OPENHCS_HEADLESS', 'false').lower() == 'true' or
+                                            os.getenv('DISPLAY') is None
+                                        )
 
-                                    if is_headless:
-                                        logger.info(f"Napari streaming disabled for step '{step.name}': running in headless environment (CI or no DISPLAY)")
-                                        continue  # Skip this streaming config
+                                        if is_headless:
+                                            logger.info(f"Napari streaming disabled for step '{step.name}': running in headless environment (CI or no DISPLAY)")
+                                            continue  # Skip this streaming config
 
-                                    napari = optional_import("napari")
-                                    if napari is None:
-                                        logger.warning(f"Napari streaming disabled for step '{step.name}': napari not installed. Install with: pip install 'openhcs[viz]' or pip install napari")
-                                        continue  # Skip this streaming config
+                                        napari = optional_import("napari")
+                                        if napari is None:
+                                            logger.warning(f"Napari streaming disabled for step '{step.name}': napari not installed. Install with: pip install 'openhcs[viz]' or pip install napari")
+                                            continue  # Skip this streaming config
 
-                            has_streaming = True
-                            # Collect visualizer info
-                            visualizer_info = {
-                                'backend': config.backend.name,
-                                'config': config
-                            }
-                            if visualizer_info not in required_visualizers:
-                                required_visualizers.append(visualizer_info)
-                        except Exception as e:
-                            logger.warning(f"Streaming disabled for step '{step.name}': {e}")
-                            continue  # Skip this streaming config
+                                has_streaming = True
+                                # Collect visualizer info
+                                visualizer_info = {
+                                    'backend': config.backend.name,
+                                    'config': config
+                                }
+                                if visualizer_info not in required_visualizers:
+                                    required_visualizers.append(visualizer_info)
+                            except Exception as e:
+                                logger.warning(f"Streaming disabled for step '{step.name}': {e}")
+                                continue  # Skip this streaming config
 
             # Set visualize flag for orchestrator if any streaming is enabled
             current_plan["visualize"] = has_streaming
@@ -684,6 +707,39 @@ class PipelineCompiler:
             if not attr_name.startswith('_'):  # Skip private attributes
                 setattr(context, attr_name, resolved_value)
 
+    @staticmethod
+    def validate_backend_compatibility(orchestrator) -> None:
+        """
+        Validate and auto-correct materialization backend for microscopes with single compatible backend.
+
+        For microscopes with only one compatible backend (e.g., OMERO → OMERO_LOCAL),
+        automatically corrects the backend if misconfigured. For microscopes with multiple
+        compatible backends, the configured backend must be explicitly compatible.
+
+        Args:
+            orchestrator: PipelineOrchestrator instance with initialized microscope_handler
+        """
+        from openhcs.core.config import VFSConfig
+        from dataclasses import replace
+
+        microscope_handler = orchestrator.microscope_handler
+        required_backend = microscope_handler.get_required_backend()
+
+        if required_backend:
+            # Microscope has single compatible backend - auto-correct if needed
+            vfs_config = orchestrator.pipeline_config.vfs_config or VFSConfig()
+
+            if vfs_config.materialization_backend != required_backend:
+                logger.warning(
+                    f"{microscope_handler.microscope_type} requires {required_backend.value} backend. "
+                    f"Auto-correcting from {vfs_config.materialization_backend.value}."
+                )
+                new_vfs_config = replace(vfs_config, materialization_backend=required_backend)
+                orchestrator.pipeline_config = replace(
+                    orchestrator.pipeline_config,
+                    vfs_config=new_vfs_config
+                )
+
     @staticmethod
     def ensure_analysis_materialization(pipeline_definition: List[AbstractStep]) -> None:
         """
@@ -714,15 +770,20 @@ class PipelineCompiler:
             # Only auto-create for intermediate steps (not final step)
             is_intermediate_step = step_index < len(pipeline_definition) - 1
 
-            if has_special_outputs and not step.step_materialization_config and is_intermediate_step:
-                # Auto-instantiate step materialization to preserve metadata coherence
-                # Use LazyStepMaterializationConfig to enable proper inheritance from global config
+            # Normalize: no config = disabled config (eliminates dual code path)
+            if not step.step_materialization_config:
+                from openhcs.config_framework.lazy_factory import LazyStepMaterializationConfig
+                step.step_materialization_config = LazyStepMaterializationConfig(enabled=False)
+
+            # Single code path: just check enabled
+            if has_special_outputs and not step.step_materialization_config.enabled and is_intermediate_step:
+                # Auto-enable materialization to preserve metadata coherence
                 from openhcs.config_framework.lazy_factory import LazyStepMaterializationConfig
                 step.step_materialization_config = LazyStepMaterializationConfig()
 
                 logger.warning(
-                    f"⚠️  Step '{step.name}' (index {step_index}) has analysis outputs but no "
-                    f"step_materialization_config. Auto-creating with defaults to preserve "
+                    f"⚠️  Step '{step.name}' (index {step_index}) has analysis outputs but lacks "
+                    f"enabled materialization config. Auto-creating with defaults to preserve "
                     f"metadata coherence (intermediate step analysis must be saved with matching images)."
                 )
                 logger.info(
@@ -767,11 +828,61 @@ class PipelineCompiler:
         if not pipeline_definition:
             raise ValueError("A valid pipeline definition (List[AbstractStep]) must be provided.")
 
+        # === BACKWARDS COMPATIBILITY PREPROCESSING ===
+        # Normalize step attributes BEFORE filtering to ensure old pickled steps have 'enabled' attribute
+        logger.debug("🔧 BACKWARDS COMPATIBILITY: Normalizing step attributes before filtering...")
+        _normalize_step_attributes(pipeline_definition)
+
+        # Filter out disabled steps at compile time (before any compilation phases)
+        original_count = len(pipeline_definition)
+        enabled_steps = []
+        for step in pipeline_definition:
+            if step.enabled:
+                enabled_steps.append(step)
+            else:
+                logger.info(f"🔧 COMPILE-TIME FILTER: Removing disabled step '{step.name}' from pipeline")
+
+        # Update pipeline_definition in-place to contain only enabled steps
+        pipeline_definition.clear()
+        pipeline_definition.extend(enabled_steps)
+
+        if original_count != len(pipeline_definition):
+            logger.info(f"🔧 COMPILE-TIME FILTER: Filtered {original_count - len(pipeline_definition)} disabled step(s), {len(pipeline_definition)} step(s) remaining")
+
+        if not pipeline_definition:
+            logger.warning("All steps were disabled. Pipeline is empty after filtering.")
+            return {
+                'pipeline_definition': pipeline_definition,
+                'compiled_contexts': {}
+            }
+
         try:
             compiled_contexts: Dict[str, ProcessingContext] = {}
             # Get multiprocessing axis values dynamically from configuration
             from openhcs.constants import MULTIPROCESSING_AXIS
-            axis_values_to_process = orchestrator.get_component_keys(MULTIPROCESSING_AXIS, axis_filter)
+
+            # CRITICAL: Resolve well_filter_config from pipeline_config if present
+            # This allows global-level well filtering to work (e.g., well_filter_config.well_filter = 1)
+            resolved_axis_filter = axis_filter
+            if orchestrator.pipeline_config and hasattr(orchestrator.pipeline_config, 'well_filter_config'):
+                well_filter_config = orchestrator.pipeline_config.well_filter_config
+                if well_filter_config and hasattr(well_filter_config, 'well_filter') and well_filter_config.well_filter is not None:
+                    from openhcs.core.utils import WellFilterProcessor
+                    available_wells = orchestrator.get_component_keys(MULTIPROCESSING_AXIS)
+                    resolved_wells = list(WellFilterProcessor.resolve_compilation_filter(
+                        well_filter_config.well_filter,
+                        available_wells
+                    ))
+                    logger.info(f"Resolved well_filter_config.well_filter={well_filter_config.well_filter} to {len(resolved_wells)} wells: {resolved_wells}")
+
+                    # If axis_filter was also provided, intersect them
+                    if axis_filter:
+                        resolved_axis_filter = [w for w in resolved_wells if w in axis_filter]
+                        logger.info(f"Intersected with axis_filter: {len(resolved_axis_filter)} wells remain")
+                    else:
+                        resolved_axis_filter = resolved_wells
+
+            axis_values_to_process = orchestrator.get_component_keys(MULTIPROCESSING_AXIS, resolved_axis_filter)
 
             if not axis_values_to_process:
                 logger.warning("No axis values found to process based on filter.")
@@ -785,7 +896,16 @@ class PipelineCompiler:
             # === ANALYSIS MATERIALIZATION AUTO-INSTANTIATION ===
             # Ensure intermediate steps with analysis outputs have step_materialization_config
             # This preserves metadata coherence (ROIs must match image structure they were created from)
-            PipelineCompiler.ensure_analysis_materialization(pipeline_definition)
+            # CRITICAL: Must be inside config_context() for lazy resolution of .enabled field
+            from openhcs.config_framework.context_manager import config_context
+            with config_context(orchestrator.pipeline_config):
+                PipelineCompiler.ensure_analysis_materialization(pipeline_definition)
+
+            # === BACKEND COMPATIBILITY VALIDATION ===
+            # Validate that configured backend is compatible with microscope
+            # For microscopes with only one compatible backend (e.g., OMERO), auto-set it
+            logger.debug("🔧 BACKEND VALIDATION: Validating backend compatibility with microscope...")
+            PipelineCompiler.validate_backend_compatibility(orchestrator)
 
             # === GLOBAL AXIS FILTER RESOLUTION ===
             # Resolve axis filters once for all axis values to ensure step-level inheritance works
