@@ -114,7 +114,7 @@ class PathPlanner:
         # Handle optional materialization and input conversion
         # Read step_materialization_config directly from step object (not step plans, which aren't populated yet)
         materialized_output_dir = None
-        if step.step_materialization_config:
+        if step.step_materialization_config and step.step_materialization_config.enabled:
             # Check if this step has well filters and if current well should be materialized
             step_axis_filters = getattr(self.ctx, 'step_axis_filters', {}).get(sid, {})
             materialization_filter = step_axis_filters.get('step_materialization_config')
@@ -136,7 +136,12 @@ class PathPlanner:
                 # No axis filter - create materialization path as normal
                 materialized_output_dir = self._build_output_path(step.step_materialization_config)
 
-        input_conversion_dir = self._get_optional_path("input_conversion_config", sid)
+        # Check if input_conversion_dir is already set by compiler (direct path)
+        # Otherwise try to calculate from input_conversion_config (legacy)
+        if "input_conversion_dir" in self.plans[sid]:
+            input_conversion_dir = Path(self.plans[sid]["input_conversion_dir"])
+        else:
+            input_conversion_dir = self._get_optional_path("input_conversion_config", sid)
 
         # Calculate main pipeline plate root for this step
         main_plate_root = self.build_output_plate_root(self.plate_path, self.cfg, is_per_step_materialization=False)
@@ -186,15 +191,8 @@ class PathPlanner:
                 'input_conversion_backend': self.vfs.materialization_backend.value
             })
 
-        # Set backend if needed
-        if getattr(step, 'input_source', None) == InputSource.PIPELINE_START:
-            self.plans[sid][READ_BACKEND] = self.vfs.materialization_backend.value
-
-            # If zarr conversion occurred, redirect input_dir to zarr store
-            if self.vfs.materialization_backend == MaterializationBackend.ZARR and pipeline:
-                first_step_plan = self.plans.get(0, {})  # Use step index 0 instead of step_id
-                if "input_conversion_dir" in first_step_plan:
-                    self.plans[sid]['input_dir'] = first_step_plan['input_conversion_dir']
+        # PIPELINE_START steps read from original input, not zarr conversion
+        # (zarr conversion only applies to normal pipeline flow, not PIPELINE_START jumps)
 
     def _get_dir(self, step: AbstractStep, i: int, pipeline: List,
                  dir_type: str, fallback: Path = None) -> Path:
@@ -335,7 +333,7 @@ class PathPlanner:
 
     def _get_input_source(self, step: AbstractStep, i: int) -> str:
         """Get input source string."""
-        if getattr(step, 'input_source', None) == InputSource.PIPELINE_START:
+        if step.input_source == InputSource.PIPELINE_START:
             return 'PIPELINE_START'
         return 'PREVIOUS_STEP'
 
@@ -384,7 +382,7 @@ class PathPlanner:
         # Collect all materialization steps with their paths and positions
         mat_steps = [
             (step, self.plans.get(i, {}).get('pipeline_position', 0), self._build_output_path(step.step_materialization_config))
-            for i, step in enumerate(pipeline) if step.step_materialization_config
+            for i, step in enumerate(pipeline) if step.step_materialization_config and step.step_materialization_config.enabled
         ]
 
         # Group by path for conflict detection
@@ -399,7 +397,6 @@ class PathPlanner:
         # Resolve materialization vs materialization conflicts
         for path_key, step_list in path_groups.items():
             if len(step_list) > 1:
-                print(f"⚠️  Materialization path collision detected for {len(step_list)} steps at: {path_key}")
                 for step, pos, path in step_list:
                     self._resolve_and_update_paths(step, pos, path, f"pos {pos}")
 
@@ -411,7 +408,6 @@ class PathPlanner:
 
         # Generate unique sub_dir name instead of calculating from paths
         original_sub_dir = materialization_config.sub_dir
-        print(f"🔍 PATH_COLLISION DEBUG: step '{step.name}' original_sub_dir = '{original_sub_dir}' (type: {type(materialization_config).__name__})")
         new_sub_dir = f"{original_sub_dir}_step{position}"
 
         # Update step materialization config with new sub_dir
@@ -427,8 +423,6 @@ class PathPlanner:
                 step_plan['materialized_output_dir'] = str(resolved_path)
                 step_plan['materialized_sub_dir'] = new_sub_dir  # Update stored sub_dir
 
-        print(f"    - step '{step.name}' ({conflict_type}) → {resolved_path}")
-
 
 
 # ===== PUBLIC API =====
@@ -475,9 +469,12 @@ class PipelinePathPlanner:
         Returns:
             Channel-specific path
         """
-        dir_part, filename = base_path.rsplit('/', 1)
+        # Use Path for cross-platform path handling (Windows uses backslashes)
+        path = Path(base_path)
+        dir_part = path.parent
+        filename = path.name
         well_id, rest = filename.split('_', 1)
-        return f"{dir_part}/{well_id}_w{dict_key}_{rest}"
+        return str(dir_part / f"{well_id}_w{dict_key}_{rest}")
 
 
 
