@@ -132,8 +132,13 @@ class OpenHCSMetadataHandler(MetadataHandler):
                 if not subdirs:
                     raise MetadataNotFoundError(f"Empty subdirectories in metadata file '{metadata_file_path}'")
 
-                # Merge all subdirectories: use first as base, combine all image_files
-                base_metadata = next(iter(subdirs.values())).copy()
+                # Use main subdirectory as base (marked with "main": true), fallback to first
+                main_subdir = next((data for data in subdirs.values() if data.get("main")), None)
+                if not main_subdir:
+                    # Fallback to first subdirectory if no main is marked
+                    main_subdir = next(iter(subdirs.values()))
+
+                base_metadata = main_subdir.copy()
                 base_metadata[FIELDS.IMAGE_FILES] = [
                     file for subdir in subdirs.values()
                     for file in subdir.get(FIELDS.IMAGE_FILES, [])
@@ -238,12 +243,8 @@ class OpenHCSMetadataHandler(MetadataHandler):
             raise ValueError(f"'{FIELDS.SOURCE_FILENAME_PARSER_NAME}' must be a non-empty string in {self.METADATA_FILENAME}")
         return parser_name
 
-    def get_image_files(self, plate_path: Union[str, Path]) -> List[str]:
-        """Get image files list from OpenHCS metadata."""
-        image_files = self._load_metadata(plate_path).get(FIELDS.IMAGE_FILES)
-        if not (isinstance(image_files, list) and all(isinstance(f, str) for f in image_files)):
-            raise ValueError(f"'{FIELDS.IMAGE_FILES}' must be a list of strings in {self.METADATA_FILENAME}")
-        return image_files
+    # Uses default get_image_files() implementation from MetadataHandler ABC
+    # (prefers workspace_mapping keys, falls back to image_files list)
 
     # Optional metadata getters
     def _get_optional_metadata_dict(self, plate_path: Union[str, Path], key: str) -> Optional[Dict[str, str]]:
@@ -378,6 +379,7 @@ class OpenHCSMetadata:
     z_indexes: Optional[Dict[str, str]]
     timepoints: Optional[Dict[str, str]]
     available_backends: Dict[str, bool]
+    workspace_mapping: Optional[Dict[str, str]] = None  # Plate-relative virtual → real path mapping
     main: Optional[bool] = None  # Indicates if this subdirectory is the primary/input subdirectory
     results_dir: Optional[str] = None  # Sibling directory containing analysis results for this subdirectory
 
@@ -444,23 +446,57 @@ class OpenHCSMetadataGenerator:
         is_main: bool = False,
         plate_root: str = None,
         sub_dir: str = None,
-        results_dir: str = None
+        results_dir: str = None,
+        skip_if_complete: bool = False,
+        allow_none_override: bool = False
     ) -> None:
-        """Create or update subdirectory-keyed OpenHCS metadata file."""
+        """Create or update subdirectory-keyed OpenHCS metadata file.
+
+        Args:
+            skip_if_complete: If True, skip update if metadata already complete (has channels)
+            allow_none_override: If True, None values override existing fields;
+                               if False (default), None values are filtered out to preserve existing fields
+        """
         plate_root_path = Path(plate_root)
         metadata_path = get_metadata_path(plate_root_path)
 
+        # Check if metadata already complete (if requested)
+        if skip_if_complete and metadata_path.exists():
+            import json
+            with open(metadata_path, 'r') as f:
+                existing = json.load(f)
+
+            subdir_data = existing.get('subdirectories', {}).get(sub_dir, {})
+            if subdir_data.get('channels'):
+                self.logger.debug(f"Metadata for {sub_dir} already complete, skipping")
+                return
+
+        # Extract metadata from current state
         current_metadata = self._extract_metadata_from_disk_state(context, output_dir, write_backend, is_main, sub_dir, results_dir)
         metadata_dict = asdict(current_metadata)
 
-        self.atomic_writer.update_subdirectory_metadata(metadata_path, sub_dir, metadata_dict)
+        # Filter None values unless override allowed
+        if not allow_none_override:
+            metadata_dict = {k: v for k, v in metadata_dict.items() if v is not None}
+
+        self.atomic_writer.merge_subdirectory_metadata(metadata_path, {sub_dir: metadata_dict})
 
 
 
     def _extract_metadata_from_disk_state(self, context: 'ProcessingContext', output_dir: str, write_backend: str, is_main: bool, sub_dir: str, results_dir: str = None) -> OpenHCSMetadata:
-        """Extract metadata reflecting current disk state after processing."""
+        """Extract metadata reflecting current disk state after processing.
+
+        CRITICAL: Extracts component metadata (channels, wells, sites, z_indexes, timepoints)
+        by parsing actual filenames in output_dir, NOT from the original input metadata cache.
+        This ensures metadata accurately reflects what was actually written, not what was in the input.
+
+        For example, if processing filters to only channels 1-2, the metadata will show only those channels.
+        """
         handler = context.microscope_handler
-        cache = context.metadata_cache or {}
+
+        # metadata_cache is always set by create_context() - fail if not present
+        if not hasattr(context, 'metadata_cache'):
+            raise RuntimeError("ProcessingContext missing metadata_cache - must be created via create_context()")
 
         actual_files = self.filemanager.list_image_files(output_dir, write_backend)
         relative_files = [f"{sub_dir}/{Path(f).name}" for f in actual_files]
@@ -472,25 +508,121 @@ class OpenHCSMetadataGenerator:
             results_path = Path(results_dir)
             relative_results_dir = results_path.name  # Just the directory name, not full path
 
+        # Extract grid_dimensions and pixel_size from input metadata
+        grid_dimensions = handler.metadata_handler._get_with_fallback('get_grid_dimensions', context.input_dir)
+        pixel_size = handler.metadata_handler._get_with_fallback('get_pixel_size', context.input_dir)
+
+        # If grid_dimensions is None (fallback), try to get it from existing metadata
+        if grid_dimensions is None:
+            try:
+                from pathlib import Path as PathlibPath
+                plate_root = PathlibPath(context.plate_path)
+                metadata_path = get_metadata_path(plate_root)
+                if metadata_path.exists():
+                    import json
+                    with open(metadata_path, 'r') as f:
+                        existing_metadata = json.load(f)
+                    # Try to get grid_dimensions from any existing subdirectory
+                    for existing_sub_dir, sub_metadata in existing_metadata.get('subdirectories', {}).items():
+                        if sub_metadata.get('grid_dimensions'):
+                            grid_dimensions = sub_metadata['grid_dimensions']
+                            self.logger.debug(f"Preserved grid_dimensions from existing subdirectory {existing_sub_dir}: {grid_dimensions}")
+                            break
+            except Exception as e:
+                self.logger.debug(f"Could not retrieve grid_dimensions from existing metadata: {e}")
+
+        # CRITICAL: Extract component metadata from actual output files by parsing filenames
+        # This ensures metadata reflects what was actually written, not the original input
+        component_metadata = self._extract_component_metadata_from_files(actual_files, handler.parser)
+
+        # Merge extracted component keys with display names from original metadata cache
+        # This preserves display names (e.g., "tl-20") while using actual output components
+        merged_metadata = self._merge_component_metadata(component_metadata, context.metadata_cache)
+
         # CRITICAL: Use AllComponents enum for cache lookups (cache is keyed by AllComponents)
         # GroupBy and AllComponents have same values but different hashes, so dict.get() fails with GroupBy
         return OpenHCSMetadata(
             microscope_handler_name=handler.microscope_type,
             source_filename_parser_name=handler.parser.__class__.__name__,
-            grid_dimensions=handler.metadata_handler._get_with_fallback('get_grid_dimensions', context.input_dir),
-            pixel_size=handler.metadata_handler._get_with_fallback('get_pixel_size', context.input_dir),
+            grid_dimensions=grid_dimensions,
+            pixel_size=pixel_size,
             image_files=relative_files,
-            channels=cache.get(AllComponents.CHANNEL),
-            wells=cache.get(AllComponents.WELL),
-            sites=cache.get(AllComponents.SITE),
-            z_indexes=cache.get(AllComponents.Z_INDEX),
-            timepoints=cache.get(AllComponents.TIMEPOINT),
+            channels=merged_metadata.get(AllComponents.CHANNEL),
+            wells=merged_metadata.get(AllComponents.WELL),
+            sites=merged_metadata.get(AllComponents.SITE),
+            z_indexes=merged_metadata.get(AllComponents.Z_INDEX),
+            timepoints=merged_metadata.get(AllComponents.TIMEPOINT),
             available_backends={write_backend: True},
+            workspace_mapping=None,  # Preserve existing - filtered out by create_metadata()
             main=is_main if is_main else None,
             results_dir=relative_results_dir
         )
 
+    def _extract_component_metadata_from_files(self, file_paths: list, parser) -> Dict[AllComponents, Optional[Dict[str, Optional[str]]]]:
+        """
+        Extract component metadata by parsing actual filenames.
 
+        Filenames are architecturally guaranteed to be properly formed by the pipeline.
+        Parser.parse_filename() is guaranteed to succeed. No defensive checks.
+
+        Args:
+            file_paths: List of image file paths (guaranteed properly formed)
+            parser: FilenameParser instance
+
+        Returns:
+            Dict mapping AllComponents to component metadata dicts (key -> display_name)
+        """
+        result = {component: {} for component in AllComponents}
+
+        for file_path in file_paths:
+            filename = Path(file_path).name
+            parsed = parser.parse_filename(filename)
+
+            # Extract each component from the parsed filename
+            for component in AllComponents:
+                component_name = component.value
+                if component_name in parsed:
+                    component_value = str(parsed[component_name])
+                    # Store with None as display name (will be merged with original metadata display names)
+                    if component_value not in result[component]:
+                        result[component][component_value] = None
+
+        # Convert empty dicts to None (no metadata for that component)
+        return {component: metadata_dict if metadata_dict else None for component, metadata_dict in result.items()}
+
+    def _merge_component_metadata(self, extracted: Dict[AllComponents, Optional[Dict[str, Optional[str]]]], cache: Dict[AllComponents, Optional[Dict[str, Optional[str]]]]) -> Dict[AllComponents, Optional[Dict[str, Optional[str]]]]:
+        """
+        Merge extracted component keys with display names from original metadata cache.
+
+        For each component:
+        - Use extracted keys (what actually exists in output)
+        - Preserve display names from cache (e.g., "tl-20" for channel "1")
+        - If no display name in cache, use None
+
+        Args:
+            extracted: Component metadata extracted from output filenames
+            cache: Original metadata cache with display names
+
+        Returns:
+            Merged metadata with actual components and preserved display names
+        """
+        result = {}
+        for component in AllComponents:
+            extracted_dict = extracted.get(component)
+            cache_dict = cache.get(component)
+
+            if extracted_dict is None:
+                result[component] = None
+            else:
+                # For each extracted key, get display name from cache if available
+                merged = {}
+                for key in extracted_dict.keys():
+                    display_name = cache_dict.get(key) if cache_dict else None
+                    merged[key] = display_name
+
+                result[component] = merged if merged else None
+
+        return result
 
 
 
@@ -599,12 +731,17 @@ class OpenHCSMicroscopeHandler(MicroscopeHandler):
 
 
     @property
-    def common_dirs(self) -> List[str]:
+    def root_dir(self) -> str:
         """
-        OpenHCS format expects images in the root of the plate folder.
-        No common subdirectories are applicable.
+        Root directory for OpenHCS is determined from metadata.
+
+        OpenHCS plates can have multiple subdirectories (e.g., "zarr", "images", ".").
+        The root_dir is determined dynamically from the main subdirectory in metadata.
+        This property returns a placeholder - actual root_dir is determined at runtime.
         """
-        return []
+        # This is determined dynamically from metadata in initialize_workspace
+        # Return empty string as placeholder (not used for virtual workspace)
+        return ""
 
     @property
     def microscope_type(self) -> str:
@@ -657,22 +794,47 @@ class OpenHCSMicroscopeHandler(MicroscopeHandler):
             # Fall back to all compatible backends if metadata reading fails
             return self.compatible_backends
 
-    def get_primary_backend(self, plate_path: Union[str, Path]) -> str:
+    def get_primary_backend(self, plate_path: Union[str, Path], filemanager: 'FileManager') -> str:
         """
         Get the primary backend name for OpenHCS plates.
 
         Uses metadata-based detection to determine the primary backend.
+        Preference hierarchy: zarr > virtual_workspace > disk
+        Registers virtual_workspace backend if needed.
+
+        Args:
+            plate_path: Input directory (may be subdirectory like zarr/)
+            filemanager: FileManager instance for backend registration
         """
-        available_backends_dict = self.metadata_handler.get_available_backends(plate_path)
+        # plate_folder must be set before calling this method
+        if self.plate_folder is None:
+            raise RuntimeError(
+                "OpenHCSHandler.determine_backend_preference: plate_folder not set. "
+                "Call determine_input_dir() or post_workspace() first."
+            )
+
+        available_backends_dict = self.metadata_handler.get_available_backends(self.plate_folder)
+
+        # Preference hierarchy: zarr > virtual_workspace > disk
+        # 1. Prefer zarr if available (best performance for large datasets)
+        if 'zarr' in available_backends_dict and available_backends_dict['zarr']:
+            return 'zarr'
+
+        # 2. Prefer virtual_workspace if available (for plates with workspace_mapping)
+        if 'virtual_workspace' in available_backends_dict and available_backends_dict['virtual_workspace']:
+            # Register virtual_workspace backend using centralized helper
+            self._register_virtual_workspace_backend(self.plate_folder, filemanager)
+            return 'virtual_workspace'
+
+        # 3. Fall back to first available backend (usually disk)
         return next(iter(available_backends_dict.keys()))
 
-    def initialize_workspace(self, plate_path: Path, workspace_path: Optional[Path], filemanager: FileManager) -> Path:
+    def initialize_workspace(self, plate_path: Path, filemanager: FileManager) -> Path:
         """
         OpenHCS format doesn't need workspace - determines the correct input subdirectory from metadata.
 
         Args:
             plate_path: Path to the original plate directory
-            workspace_path: Optional workspace path (ignored for OpenHCS)
             filemanager: FileManager instance for file operations
 
         Returns:
@@ -688,6 +850,14 @@ class OpenHCSMicroscopeHandler(MicroscopeHandler):
         main_subdir = self.metadata_handler.determine_main_subdirectory(plate_path)
         input_dir = plate_path / main_subdir
 
+        # Check if workspace_mapping exists in metadata - if so, register virtual workspace backend
+        metadata_dict = self.metadata_handler._load_metadata_dict(plate_path)
+        subdir_metadata = metadata_dict.get(FIELDS.SUBDIRECTORIES, {}).get(main_subdir, {})
+
+        if subdir_metadata.get('workspace_mapping'):
+            # Register virtual_workspace backend using centralized helper
+            self._register_virtual_workspace_backend(plate_path, filemanager)
+
         # Verify the subdirectory exists - fail-loud if missing
         if not filemanager.is_dir(str(input_dir), Backend.DISK.value):
             raise FileNotFoundError(
@@ -715,14 +885,14 @@ class OpenHCSMicroscopeHandler(MicroscopeHandler):
             logger.debug(f"OpenHCSHandler: plate_folder set to {self.plate_folder} during _prepare_workspace.")
         return workspace_path
 
-    def post_workspace(self, workspace_path: Union[str, Path], filemanager: FileManager, width: int = 3):
+    def post_workspace(self, plate_path: Union[str, Path], filemanager: FileManager, skip_preparation: bool = False) -> Path:
         """
-        Hook called after workspace symlink creation.
+        Hook called after virtual workspace mapping creation.
         For OpenHCS, this ensures the plate_folder is set (if not already) which allows
-        the parser to be loaded using this workspace_path. It then calls the base
+        the parser to be loaded using this plate_path. It then calls the base
         implementation which handles filename normalization using the loaded parser.
         """
-        current_plate_folder = Path(workspace_path)
+        current_plate_folder = Path(plate_path)
         if self.plate_folder is None:
             logger.info(f"OpenHCSHandler.post_workspace: Setting plate_folder to {current_plate_folder}.")
             self.plate_folder = current_plate_folder
@@ -740,7 +910,7 @@ class OpenHCSMicroscopeHandler(MicroscopeHandler):
 
         logger.info(f"OpenHCSHandler (plate: {self.plate_folder}): Files are expected to be pre-normalized. "
                      "Superclass post_workspace will run with the dynamically loaded parser.")
-        return super().post_workspace(workspace_path, filemanager, width)
+        return super().post_workspace(plate_path, filemanager, skip_preparation)
 
     # The following methods from MicroscopeHandler delegate to `self.parser`.
     # The `parser` property will ensure the correct, dynamically loaded parser is used.
