@@ -912,22 +912,30 @@ class LogViewerWindow(QMainWindow):
         import zmq
         import pickle
 
-        logger.debug("Scanning for running ZMQ/Napari servers...")
+        logger.debug("Scanning for running ZMQ/streaming servers...")
         discovered_logs = []
 
-        # Common ports to scan
-        zmq_execution_ports = [7777]  # Default ZMQ execution server port
-        napari_ports = [5555 + i for i in range(5)]  # Scan Napari ports 5555-5559
+        # Scan all streaming ports using current global config
+        # This ensures we find viewers launched with custom ports
+        from openhcs.core.config import get_all_streaming_ports
+        ports_to_scan = get_all_streaming_ports(num_ports_per_type=10)  # Uses global config by default
 
         def ping_server(port: int) -> dict:
             """Ping a server and return pong response, or None if no response."""
-            control_port = port + 1000
+            from openhcs.constants.constants import CONTROL_PORT_OFFSET
+            from openhcs.runtime.zmq_base import get_zmq_transport_url, get_default_transport_mode
+
+            control_port = port + CONTROL_PORT_OFFSET
             try:
                 context = zmq.Context()
                 socket = context.socket(zmq.REQ)
                 socket.setsockopt(zmq.LINGER, 0)
                 socket.setsockopt(zmq.RCVTIMEO, 1000)  # 1 second timeout (servers may be busy)
-                socket.connect(f"tcp://localhost:{control_port}")
+
+                # Use transport mode-aware URL (IPC or TCP)
+                transport_mode = get_default_transport_mode()
+                control_url = get_zmq_transport_url(control_port, transport_mode, 'localhost')
+                socket.connect(control_url)
 
                 # Send ping
                 socket.send(pickle.dumps({'type': 'ping'}))
@@ -944,25 +952,16 @@ class LogViewerWindow(QMainWindow):
                 logger.debug(f"Port {port} no response: {e}")
                 return None
 
-        # Scan ZMQ execution servers
-        for port in zmq_execution_ports:
+        # Scan all ports (execution server + all streaming types)
+        for port in ports_to_scan:
             pong = ping_server(port)
             if pong and pong.get('log_file_path'):
                 log_path = Path(pong['log_file_path'])
                 if log_path.exists():
                     log_info = classify_log_file(log_path, None, False)
                     discovered_logs.append(log_info)
-                    logger.debug(f"Discovered ZMQ server log: {log_path}")
-
-        # Scan Napari viewers
-        for port in napari_ports:
-            pong = ping_server(port)
-            if pong and pong.get('viewer') == 'napari' and pong.get('log_file_path'):
-                log_path = Path(pong['log_file_path'])
-                if log_path.exists():
-                    log_info = classify_log_file(log_path, None, False)
-                    discovered_logs.append(log_info)
-                    logger.debug(f"Discovered Napari viewer log: {log_path}")
+                    viewer_type = pong.get('viewer', 'ZMQ server')
+                    logger.debug(f"Discovered {viewer_type} log: {log_path}")
 
         return discovered_logs
 
