@@ -17,8 +17,8 @@ from pathlib import Path
 
 from PyQt6.QtWidgets import (
     QWidget, QVBoxLayout, QHBoxLayout, QPushButton, QListWidget,
-    QListWidgetItem, QLabel, QProgressBar,
-    QSplitter
+    QListWidgetItem, QLabel,
+    QSplitter, QApplication
 )
 from PyQt6.QtCore import Qt, pyqtSignal, pyqtSlot
 from PyQt6.QtGui import QFont
@@ -62,7 +62,7 @@ class PlateManagerWidget(QWidget):
     subprocess_log_stopped = pyqtSignal()
     clear_subprocess_logs = pyqtSignal()
 
-    # Progress update signals (thread-safe UI updates)
+    # Progress update signals (thread-safe UI updates) - routed to status bar
     progress_started = pyqtSignal(int)  # max_value
     progress_updated = pyqtSignal(int)  # current_value
     progress_finished = pyqtSignal()
@@ -71,6 +71,10 @@ class PlateManagerWidget(QWidget):
     compilation_error = pyqtSignal(str, str)  # plate_name, error_message
     initialization_error = pyqtSignal(str, str)  # plate_name, error_message
     execution_error = pyqtSignal(str)  # error_message
+
+    # Internal signals for thread-safe completion handling
+    _execution_complete_signal = pyqtSignal(dict, list)  # result, ready_items
+    _execution_error_signal = pyqtSignal(str)  # error_msg
     
     def __init__(self, file_manager: FileManager, service_adapter,
                  color_scheme: Optional[PyQt6ColorScheme] = None, parent=None):
@@ -112,15 +116,49 @@ class PlateManagerWidget(QWidget):
         self.plate_list: Optional[QListWidget] = None
         self.buttons: Dict[str, QPushButton] = {}
         self.status_label: Optional[QLabel] = None
-        self.progress_bar: Optional[QProgressBar] = None
         
         # Setup UI
         self.setup_ui()
         self.setup_connections()
         self.update_button_states()
-        
+
+        # Connect internal signals for thread-safe completion handling
+        self._execution_complete_signal.connect(self._on_execution_complete)
+        self._execution_error_signal.connect(self._on_execution_error)
+
         logger.debug("Plate manager widget initialized")
 
+    def cleanup(self):
+        """Cleanup resources before widget destruction."""
+        logger.info("🧹 Cleaning up PlateManagerWidget resources...")
+
+        # Disconnect and cleanup ZMQ client if it exists
+        if self.zmq_client is not None:
+            try:
+                logger.info("🧹 Disconnecting ZMQ client...")
+                self.zmq_client.disconnect()
+            except Exception as e:
+                logger.warning(f"Error disconnecting ZMQ client during cleanup: {e}")
+            finally:
+                self.zmq_client = None
+
+        # Terminate any running subprocess
+        if self.current_process is not None and self.current_process.poll() is None:
+            try:
+                logger.info("🧹 Terminating running subprocess...")
+                self.current_process.terminate()
+                self.current_process.wait(timeout=2)
+            except Exception as e:
+                logger.warning(f"Error terminating subprocess during cleanup: {e}")
+                try:
+                    self.current_process.kill()
+                except:
+                    pass
+            finally:
+                self.current_process = None
+
+        logger.info("✅ PlateManagerWidget cleanup completed")
+
     # ========== UI Setup ==========
 
     def setup_ui(self):
@@ -185,11 +223,6 @@ class PlateManagerWidget(QWidget):
         button_panel = self.create_button_panel()
         splitter.addWidget(button_panel)
 
-        # Progress bar (below splitter)
-        self.progress_bar = QProgressBar()
-        self.progress_bar.setVisible(False)
-        layout.addWidget(self.progress_bar)
-
         # Set splitter proportions - make button panel much smaller
         splitter.setSizes([400, 80])
     
@@ -223,7 +256,7 @@ class PlateManagerWidget(QWidget):
             ("Compile", "compile_plate", "Compile plate pipelines"),
             ("Run", "run_plate", "Run/Stop plate execution"),
             ("Code", "code_plate", "Generate Python code"),
-            ("Meta", "view_metadata", "View plate metadata"),
+            ("Viewer", "view_metadata", "View plate metadata"),
         ]
         
         # Create buttons in rows
@@ -469,17 +502,32 @@ class PlateManagerWidget(QWidget):
                 ensure_global_config_context(GlobalPipelineConfig, self.global_config)
                 return orchestrator.initialize()
 
-            await asyncio.get_event_loop().run_in_executor(
-                None,
-                initialize_with_context
-            )
+            try:
+                await asyncio.get_event_loop().run_in_executor(
+                    None,
+                    initialize_with_context
+                )
 
-            self.orchestrators[plate_path] = orchestrator
-            self.orchestrator_state_changed.emit(plate_path, "READY")
+                self.orchestrators[plate_path] = orchestrator
+                self.orchestrator_state_changed.emit(plate_path, "READY")
 
-            if not self.selected_plate_path:
-                self.selected_plate_path = plate_path
-                self.plate_selected.emit(plate_path)
+                if not self.selected_plate_path:
+                    self.selected_plate_path = plate_path
+                    self.plate_selected.emit(plate_path)
+
+            except Exception as e:
+                logger.error(f"Failed to initialize plate {plate_path}: {e}", exc_info=True)
+                # Create a failed orchestrator to track the error state
+                failed_orchestrator = PipelineOrchestrator(
+                    plate_path=plate_path,
+                    storage_registry=self.file_manager.registry
+                )
+                failed_orchestrator._state = OrchestratorState.INIT_FAILED
+                self.orchestrators[plate_path] = failed_orchestrator
+                # Emit signal to update UI with failed state
+                self.orchestrator_state_changed.emit(plate_path, OrchestratorState.INIT_FAILED.value)
+                # Show error dialog
+                self.initialization_error.emit(plate['name'], str(e))
 
             self.progress_updated.emit(i + 1)
 
@@ -490,7 +538,15 @@ class PlateManagerWidget(QWidget):
         ])
 
         self.progress_finished.emit()
-        self.status_message.emit(f"Initialized {len(selected_items)} plate(s)")
+
+        # Count successes and failures
+        success_count = len([p for p in selected_items if self.orchestrators.get(p['path']) and self.orchestrators[p['path']].state == OrchestratorState.READY])
+        error_count = len([p for p in selected_items if self.orchestrators.get(p['path']) and self.orchestrators[p['path']].state == OrchestratorState.INIT_FAILED])
+
+        if error_count == 0:
+            self.status_message.emit(f"Successfully initialized {success_count} plate(s)")
+        else:
+            self.status_message.emit(f"Initialized {success_count} plate(s), {error_count} error(s)")
     
     # Additional action methods would be implemented here following the same pattern...
     # (compile_plate, run_plate, code_plate, view_metadata, edit_config)
@@ -834,173 +890,26 @@ class PlateManagerWidget(QWidget):
         self.update_button_states()
     
     async def action_run_plate(self):
-        """Handle Run Plate button - execute compiled plates."""
+        """Handle Run Plate button - execute compiled plates using ZMQ."""
         selected_items = self.get_selected_plates()
         if not selected_items:
-            self.service_adapter.show_error_dialog("No plates selected to run.")
+            # Use signal for thread-safe error reporting from async context
+            self.execution_error.emit("No plates selected to run.")
             return
 
         ready_items = [item for item in selected_items if item.get('path') in self.plate_compiled_data]
         if not ready_items:
-            self.service_adapter.show_error_dialog("Selected plates are not compiled. Please compile first.")
+            # Use signal for thread-safe error reporting from async context
+            self.execution_error.emit("Selected plates are not compiled. Please compile first.")
             return
 
-        # Check if ZMQ execution is enabled (default: True)
-        import os
-        use_zmq = os.environ.get('OPENHCS_USE_ZMQ_EXECUTION', 'true').lower() in ('true', '1', 'yes')
+        await self._run_plates_zmq(ready_items)
 
-        if use_zmq:
-            await self._run_plates_zmq(ready_items)
-        else:
-            await self._run_plates_subprocess(ready_items)
-
-    async def _run_plates_subprocess(self, ready_items):
-        """Run plates using legacy subprocess runner (deprecated)."""
-        import warnings
-        warnings.warn(
-            "Subprocess runner is deprecated. Set OPENHCS_USE_ZMQ_EXECUTION=true to use ZMQ execution.",
-            DeprecationWarning,
-            stacklevel=2
-        )
-
-        try:
-            # Use subprocess approach like Textual TUI
-            logger.debug("Using subprocess approach for clean isolation")
-
-            plate_paths_to_run = [item['path'] for item in ready_items]
-
-            # Pass both pipeline definition and pre-compiled contexts to subprocess
-            pipeline_data = {}
-            effective_configs = {}
-            for plate_path in plate_paths_to_run:
-                compiled_data = self.plate_compiled_data[plate_path]
-                pipeline_data[plate_path] = {
-                    'pipeline_definition': compiled_data['execution_pipeline'],  # Use execution pipeline (stripped)
-                    'compiled_contexts': compiled_data['compiled_contexts']      # Pre-compiled contexts
-                }
-
-                # Get effective config for this plate (includes pipeline config if set)
-                # For subprocess execution, we need the merged config since subprocess doesn't use dual-axis resolver
-                if plate_path in self.orchestrators:
-                    # Get merged config from orchestrator (this resolves lazy values)
-                    effective_configs[plate_path] = self.orchestrators[plate_path].get_effective_config()
-                else:
-                    effective_configs[plate_path] = self.global_config
-
-            logger.info(f"Starting subprocess for {len(plate_paths_to_run)} plates")
-
-            # Clear subprocess logs before starting new execution
-            self.clear_subprocess_logs.emit()
-
-            # Create data file for subprocess
-            data_file = tempfile.NamedTemporaryFile(mode='wb', delete=False, suffix='.pkl')
-
-            # Generate unique ID for this subprocess
-            import time
-            subprocess_timestamp = int(time.time())
-            plate_names = [Path(path).name for path in plate_paths_to_run]
-            unique_id = f"plates_{'_'.join(plate_names[:2])}_{subprocess_timestamp}"
-
-            # Build subprocess log name using log utilities
-            from openhcs.core.log_utils import get_current_log_file_path
-            try:
-                tui_log_path = get_current_log_file_path()
-                if tui_log_path.endswith('.log'):
-                    tui_base = tui_log_path[:-4]  # Remove .log extension
-                else:
-                    tui_base = tui_log_path
-                log_file_base = f"{tui_base}_subprocess_{subprocess_timestamp}"
-            except RuntimeError:
-                # Fallback if no main log found
-                log_dir = Path.home() / ".local" / "share" / "openhcs" / "logs"
-                log_dir.mkdir(parents=True, exist_ok=True)
-                log_file_base = str(log_dir / f"pyqt_gui_subprocess_{subprocess_timestamp}")
-
-            # Pickle data for subprocess
-            subprocess_data = {
-                'plate_paths': plate_paths_to_run,
-                'pipeline_data': pipeline_data,
-                'global_config': self.global_config,      # Fallback global config
-                'effective_configs': effective_configs    # Per-plate effective configs (includes pipeline config)
-            }
-
-            # Resolve all lazy configurations to concrete values before pickling
-            from openhcs.config_framework.lazy_factory import resolve_lazy_configurations_for_serialization
-            resolved_subprocess_data = resolve_lazy_configurations_for_serialization(subprocess_data)
-
-            # Write pickle data
-            def _write_pickle_data():
-                import dill as pickle
-                with open(data_file.name, 'wb') as f:
-                    pickle.dump(resolved_subprocess_data, f)
-                data_file.close()
-
-            # Write pickle data in executor (works in Qt thread)
-            import asyncio
-            loop = asyncio.get_event_loop()
-            await loop.run_in_executor(None, _write_pickle_data)
-
-            logger.debug(f"Created data file: {data_file.name}")
-
-            # Create subprocess
-            subprocess_script = Path(__file__).parent.parent.parent / "textual_tui" / "subprocess_runner.py"
-
-            # Generate actual log file path that subprocess will create
-            actual_log_file_path = f"{log_file_base}_{unique_id}.log"
-            logger.debug(f"Log file base: {log_file_base}")
-            logger.debug(f"Unique ID: {unique_id}")
-            logger.debug(f"Actual log file: {actual_log_file_path}")
-
-            # Store log file path for monitoring
-            self.log_file_path = actual_log_file_path
-            self.log_file_position = 0
-
-            logger.debug(f"Subprocess command: {sys.executable} {subprocess_script} {data_file.name} {log_file_base} {unique_id}")
-
-            # Create subprocess
-            def _create_subprocess():
-                return subprocess.Popen([
-                    sys.executable, str(subprocess_script),
-                    data_file.name, log_file_base, unique_id
-                ],
-                stdout=subprocess.DEVNULL,
-                stderr=subprocess.DEVNULL,
-                text=True,
-                )
-
-            # Create subprocess in executor (works in Qt thread)
-            import asyncio
-            loop = asyncio.get_event_loop()
-            self.current_process = await loop.run_in_executor(None, _create_subprocess)
-
-            logger.info(f"Subprocess started with PID: {self.current_process.pid}")
-
-            # Emit signal for log viewer to start monitoring
-            self.subprocess_log_started.emit(log_file_base)
-
-            # Update orchestrator states to show running state
-            for plate in ready_items:
-                plate_path = plate['path']
-                if plate_path in self.orchestrators:
-                    self.orchestrators[plate_path]._state = OrchestratorState.EXECUTING
-
-            self.execution_state = "running"
-            self.status_message.emit(f"Running {len(ready_items)} plate(s) in subprocess...")
-            self.update_button_states()
-
-            # Start monitoring
-            await self._start_monitoring()
-
-        except Exception as e:
-            logger.error(f"Failed to start plate execution: {e}", exc_info=True)
-            self.service_adapter.show_error_dialog(f"Failed to start execution: {e}")
-            self.execution_state = "idle"
-            self.update_button_states()
-    
     async def _run_plates_zmq(self, ready_items):
         """Run plates using ZMQ execution client (recommended)."""
         try:
             from openhcs.runtime.zmq_execution_client import ZMQExecutionClient
+            import asyncio
 
             plate_paths_to_run = [item['path'] for item in ready_items]
             logger.info(f"Starting ZMQ execution for {len(plate_paths_to_run)} plates")
@@ -1008,7 +917,24 @@ class PlateManagerWidget(QWidget):
             # Clear subprocess logs before starting new execution
             self.clear_subprocess_logs.emit()
 
-            # Create ZMQ client (persistent mode - server stays alive like Napari)
+            # Get event loop (needed for all async operations)
+            loop = asyncio.get_event_loop()
+
+            # Always create a fresh client for each execution to avoid state conflicts
+            # Clean up old client if it exists
+            if self.zmq_client is not None:
+                logger.info("🧹 Disconnecting previous ZMQ client")
+                try:
+                    def _disconnect_old():
+                        self.zmq_client.disconnect()
+                    await loop.run_in_executor(None, _disconnect_old)
+                except Exception as e:
+                    logger.warning(f"Error disconnecting old client: {e}")
+                finally:
+                    self.zmq_client = None
+
+            # Create new ZMQ client (persistent mode - server stays alive)
+            logger.info("🔌 Creating new ZMQ client")
             self.zmq_client = ZMQExecutionClient(
                 port=7777,
                 persistent=True,  # Server persists across executions
@@ -1019,14 +945,12 @@ class PlateManagerWidget(QWidget):
             def _connect():
                 return self.zmq_client.connect(timeout=15)
 
-            import asyncio
-            loop = asyncio.get_event_loop()
             connected = await loop.run_in_executor(None, _connect)
 
             if not connected:
                 raise RuntimeError("Failed to connect to ZMQ execution server")
 
-            logger.info("Connected to ZMQ execution server")
+            logger.info("✅ Connected to ZMQ execution server")
 
             # Update orchestrator states to show running state
             for plate in ready_items:
@@ -1062,49 +986,50 @@ class PlateManagerWidget(QWidget):
 
                 logger.info(f"Executing plate: {plate_path}")
 
-                # Execute via ZMQ (in executor to avoid blocking UI)
+                # Submit pipeline via ZMQ (non-blocking - returns immediately)
                 # Send original definition pipeline - server will compile it
-                def _execute():
-                    return self.zmq_client.execute_pipeline(
+                def _submit():
+                    return self.zmq_client.submit_pipeline(
                         plate_id=str(plate_path),
                         pipeline_steps=definition_pipeline,
                         global_config=global_config_to_send,
                         pipeline_config=pipeline_config
                     )
 
-                response = await loop.run_in_executor(None, _execute)
+                response = await loop.run_in_executor(None, _submit)
 
                 # Track execution ID for cancellation
                 if response.get('execution_id'):
                     self.current_execution_id = response['execution_id']
 
-                logger.info(f"Plate {plate_path} execution response: {response.get('status')}")
+                logger.info(f"Plate {plate_path} submission response: {response.get('status')}")
 
-                # Handle different response statuses
+                # Handle submission response (not completion - that comes via progress callback)
                 status = response.get('status')
-                if status == 'cancelled':
-                    # Cancellation is expected, not an error - just log it
-                    logger.info(f"Plate {plate_path} execution was cancelled")
-                    self.status_message.emit(f"Execution cancelled for {plate_path}")
-                elif status != 'complete':
-                    # Actual error - show error dialog
+                if status == 'accepted':
+                    # Execution submitted successfully - it's now running in background
+                    logger.info(f"Plate {plate_path} execution submitted successfully, ID={response.get('execution_id')}")
+                    self.status_message.emit(f"Executing {plate_path}... (check progress below)")
+
+                    # Start polling for completion in background (non-blocking)
+                    execution_id = response.get('execution_id')
+                    if execution_id:
+                        self._start_completion_poller(execution_id, plate_paths_to_run, ready_items)
+                else:
+                    # Submission failed - handle error
                     error_msg = response.get('message', 'Unknown error')
-                    logger.error(f"Plate {plate_path} execution failed: {error_msg}")
-                    self.service_adapter.show_error_dialog(f"Execution failed for {plate_path}: {error_msg}")
+                    logger.error(f"Plate {plate_path} submission failed: {error_msg}")
+                    self.execution_error.emit(f"Submission failed for {plate_path}: {error_msg}")
 
-            # Execution complete
-            self.execution_state = "idle"
-            self.current_execution_id = None
-            self.status_message.emit(f"Completed {len(ready_items)} plate(s)")
-
-            # Update orchestrator states
-            for plate in ready_items:
-                plate_path = plate['path']
-                if plate_path in self.orchestrators:
-                    self.orchestrators[plate_path]._state = OrchestratorState.COMPLETED
-                    self.orchestrator_state_changed.emit(plate_path, OrchestratorState.COMPLETED.value)
-
-            self.update_button_states()
+                    # Reset state on submission failure
+                    self.execution_state = "idle"
+                    self.current_execution_id = None
+                    for plate in ready_items:
+                        plate_path = plate['path']
+                        if plate_path in self.orchestrators:
+                            self.orchestrators[plate_path]._state = OrchestratorState.READY
+                            self.orchestrator_state_changed.emit(plate_path, OrchestratorState.READY.value)
+                    self.update_button_states()
 
         except Exception as e:
             logger.error(f"Failed to execute plates via ZMQ: {e}", exc_info=True)
@@ -1112,28 +1037,100 @@ class PlateManagerWidget(QWidget):
             self.execution_error.emit(f"Failed to execute: {e}")
             self.execution_state = "idle"
 
-        finally:
-            # Always disconnect from server, even if execution failed
+            # Disconnect client on error
             if self.zmq_client is not None:
                 try:
                     def _disconnect():
                         self.zmq_client.disconnect()
-
                     await loop.run_in_executor(None, _disconnect)
                 except Exception as disconnect_error:
                     logger.warning(f"Failed to disconnect ZMQ client: {disconnect_error}")
                 finally:
                     self.zmq_client = None
+
             self.current_execution_id = None
             self.update_button_states()
 
-            # Cleanup ZMQ client
-            if hasattr(self, 'zmq_client') and self.zmq_client:
+    def _start_completion_poller(self, execution_id, plate_paths, ready_items):
+        """
+        Start background thread to poll for execution completion (non-blocking).
+
+        Args:
+            execution_id: Execution ID to poll
+            plate_paths: List of plate paths being executed
+            ready_items: List of plate items being executed
+        """
+        import threading
+
+        def poll_completion():
+            """Poll for completion in background thread."""
+            try:
+                # Wait for completion (blocking in this thread, but not UI thread)
+                result = self.zmq_client.wait_for_completion(execution_id)
+
+                # Emit completion signal (thread-safe via Qt signal)
+                self._execution_complete_signal.emit(result, ready_items)
+
+            except Exception as e:
+                logger.error(f"Error polling for completion: {e}", exc_info=True)
+                # Emit error signal (thread-safe via Qt signal)
+                self._execution_error_signal.emit(str(e))
+
+        # Start polling thread
+        thread = threading.Thread(target=poll_completion, daemon=True)
+        thread.start()
+
+    def _on_execution_complete(self, result, ready_items):
+        """Handle execution completion (called from main thread via signal)."""
+        try:
+            status = result.get('status')
+            logger.info(f"Execution completed with status: {status}")
+
+            if status == 'complete':
+                self.status_message.emit(f"Completed {len(ready_items)} plate(s)")
+            elif status == 'cancelled':
+                self.status_message.emit(f"Execution cancelled")
+            else:
+                error_msg = result.get('message', 'Unknown error')
+                self.execution_error.emit(f"Execution failed: {error_msg}")
+
+            # Disconnect ZMQ client on completion
+            if self.zmq_client is not None:
                 try:
+                    logger.info("Disconnecting ZMQ client after execution completion")
                     self.zmq_client.disconnect()
-                except:
-                    pass
-                self.zmq_client = None
+                except Exception as disconnect_error:
+                    logger.warning(f"Failed to disconnect ZMQ client: {disconnect_error}")
+                finally:
+                    self.zmq_client = None
+
+            # Update state
+            self.execution_state = "idle"
+            self.current_execution_id = None
+
+            # Update orchestrator states
+            # Note: orchestrator_state_changed signal triggers on_orchestrator_state_changed()
+            # which calls update_plate_list(), so we don't need to call update_button_states() here
+            # (calling it here causes recursive repaint and crashes)
+            for plate in ready_items:
+                plate_path = plate['path']
+                if plate_path in self.orchestrators:
+                    if status == 'complete':
+                        self.orchestrators[plate_path]._state = OrchestratorState.COMPLETED
+                        self.orchestrator_state_changed.emit(plate_path, OrchestratorState.COMPLETED.value)
+                    else:
+                        self.orchestrators[plate_path]._state = OrchestratorState.READY
+                        self.orchestrator_state_changed.emit(plate_path, OrchestratorState.READY.value)
+
+        except Exception as e:
+            logger.error(f"Error handling execution completion: {e}", exc_info=True)
+
+    def _on_execution_error(self, error_msg):
+        """Handle execution error (called from main thread via signal)."""
+        self.execution_error.emit(f"Execution error: {error_msg}")
+        self.execution_state = "idle"
+        self.current_execution_id = None
+        self.update_button_states()
 
     def _on_zmq_progress(self, message):
         """
@@ -1170,77 +1167,63 @@ class PlateManagerWidget(QWidget):
         self.status_message.emit(message)
 
     async def action_stop_execution(self):
-        """Handle Stop Execution - cancel ZMQ execution or terminate subprocess."""
-        logger.info("🛑 Stop button pressed.")
-        self.status_message.emit("Terminating execution...")
+        """Handle Stop Execution - cancel ZMQ execution or terminate subprocess.
+
+        First click: Graceful shutdown, button changes to "Force Kill"
+        Second click: Force shutdown
+
+        Uses EXACT same code path as ZMQ browser quit button.
+        """
+        logger.info("🛑🛑🛑 action_stop_execution CALLED")
+        logger.info(f"🛑 execution_state: {self.execution_state}")
+        logger.info(f"🛑 zmq_client: {self.zmq_client}")
+        logger.info(f"🛑 Button text: {self.buttons['run_plate'].text()}")
+
+        # Check if this is a force kill (button text is "Force Kill")
+        is_force_kill = self.buttons["run_plate"].text() == "Force Kill"
 
         # Check if using ZMQ execution
         if self.zmq_client:
-            try:
-                logger.info("🛑 Requesting graceful cancellation via ZMQ...")
+            port = self.zmq_client.port
 
-                import asyncio
-                loop = asyncio.get_event_loop()
-
-                # Cancel specific execution if we have an ID
-                if self.current_execution_id:
-                    logger.info(f"🛑 Cancelling execution {self.current_execution_id}")
-
-                    def _cancel():
-                        return self.zmq_client.cancel_execution(self.current_execution_id)
-
-                    response = await loop.run_in_executor(None, _cancel)
-
-                    if response.get('status') == 'ok':
-                        logger.info("🛑 Cancellation request accepted, waiting for graceful shutdown...")
-                        self.status_message.emit("Cancellation requested, waiting...")
-
-                        # Wait for graceful cancellation with timeout
-                        timeout = 5  # seconds
-                        start_time = asyncio.get_event_loop().time()
-
-                        while (asyncio.get_event_loop().time() - start_time) < timeout:
-                            # Check if execution is still running
-                            def _check_status():
-                                return self.zmq_client.get_status(self.current_execution_id)
-
-                            status_response = await loop.run_in_executor(None, _check_status)
-
-                            if status_response.get('status') == 'error':
-                                # Execution no longer exists (completed or cancelled)
-                                logger.info("🛑 Execution completed/cancelled gracefully")
-                                break
-
-                            await asyncio.sleep(0.5)
-                        else:
-                            # Timeout reached - execution still running
-                            logger.warning("🛑 Graceful cancellation timeout - execution may still be running")
-                            self.status_message.emit("Cancellation timeout - execution may still be running")
-                    else:
-                        logger.warning(f"🛑 Cancellation failed: {response.get('message')}")
-                        self.status_message.emit(f"Cancellation failed: {response.get('message')}")
-
-                # Disconnect client
-                def _disconnect():
-                    self.zmq_client.disconnect()
-
-                await loop.run_in_executor(None, _disconnect)
-
-                self.zmq_client = None
-                self.current_execution_id = None
-                self.execution_state = "idle"
-
-                # Update orchestrator states
-                for orchestrator in self.orchestrators.values():
-                    if orchestrator.state == OrchestratorState.EXECUTING:
-                        orchestrator._state = OrchestratorState.COMPILED
-
-                self.status_message.emit("Execution cancelled by user")
+            # Change button to "Force Kill" IMMEDIATELY (before any async operations)
+            if not is_force_kill:
+                logger.info(f"🛑 Stop button pressed - changing to Force Kill")
+                self.execution_state = "force_kill_ready"
                 self.update_button_states()
+                # Force immediate UI update
+                QApplication.processEvents()
 
-            except Exception as e:
-                logger.error(f"🛑 Error cancelling ZMQ execution: {e}")
-                self.service_adapter.show_error_dialog(f"Failed to cancel execution: {e}")
+            # Use EXACT same code path as ZMQ browser quit button
+            import threading
+
+            def kill_server():
+                from openhcs.runtime.zmq_base import ZMQClient
+                try:
+                    graceful = not is_force_kill
+                    logger.info(f"🛑 {'Gracefully' if graceful else 'Force'} killing server on port {port}...")
+                    success = ZMQClient.kill_server_on_port(port, graceful=graceful)
+
+                    if success:
+                        logger.info(f"✅ Successfully {'quit' if graceful else 'force killed'} server on port {port}")
+                        # Emit signal to update UI on main thread
+                        self._execution_complete_signal.emit(
+                            {'status': 'cancelled'},
+                            []  # No ready_items needed for cancellation
+                        )
+                    else:
+                        logger.warning(f"❌ Failed to {'quit' if graceful else 'force kill'} server on port {port}")
+                        self._execution_error_signal.emit(f"Failed to stop execution on port {port}")
+
+                except Exception as e:
+                    logger.error(f"❌ Error stopping server on port {port}: {e}")
+                    self._execution_error_signal.emit(f"Error stopping execution: {e}")
+
+            # Run in background thread (same as ZMQ browser)
+            thread = threading.Thread(target=kill_server, daemon=True)
+            thread.start()
+
+            return
 
         elif self.current_process and self.current_process.poll() is None:  # Still running subprocess
             try:
@@ -1298,8 +1281,6 @@ class PlateManagerWidget(QWidget):
                 self.status_message.emit("Execution terminated by user")
                 self.update_button_states()
                 self.subprocess_log_stopped.emit()
-        else:
-            self.service_adapter.show_info_dialog("No execution is currently running.")
     
     def action_code_plate(self):
         """Generate Python code for selected plates and their pipelines (Tier 3)."""
@@ -1428,6 +1409,11 @@ class PlateManagerWidget(QWidget):
         """Handle edited orchestrator code and update UI state (same logic as Textual TUI)."""
         logger.debug("Orchestrator code edited, processing changes...")
         try:
+            # Ensure pipeline editor window is open before processing orchestrator code
+            main_window = self._find_main_window()
+            if main_window and hasattr(main_window, 'show_pipeline_editor'):
+                main_window.show_pipeline_editor()
+
             # CRITICAL FIX: Execute code with lazy dataclass constructor patching to preserve None vs concrete distinction
             namespace = {}
             with self._patch_lazy_constructors():
@@ -1517,7 +1503,6 @@ class PlateManagerWidget(QWidget):
 
                 # Trigger UI refresh
                 self.pipeline_data_changed.emit()
-                self.service_adapter.show_info_dialog("Orchestrator configuration updated successfully")
 
             else:
                 raise ValueError("No valid assignments found in edited code")
@@ -1609,6 +1594,8 @@ class PlateManagerWidget(QWidget):
                     status_indicators.append("🔄 Running")
                 elif orchestrator.state == OrchestratorState.COMPLETED:
                     status_indicators.append("✅ Complete")
+                elif orchestrator.state == OrchestratorState.INIT_FAILED:
+                    status_indicators.append("🚫 Init Failed")
                 elif orchestrator.state == OrchestratorState.COMPILE_FAILED:
                     status_indicators.append("❌ Compile Failed")
                 elif orchestrator.state == OrchestratorState.EXEC_FAILED:
@@ -1690,21 +1677,32 @@ class PlateManagerWidget(QWidget):
         self.buttons["view_metadata"].setEnabled(has_initialized and not is_running)
 
         # Run button - enabled if plates are compiled or if currently running (for stop)
-        if is_running:
+        if self.execution_state == "stopping":
+            # Stopping state - keep button as "Stop" but disable it
+            self.buttons["run_plate"].setEnabled(False)
+            self.buttons["run_plate"].setText("Stop")
+        elif self.execution_state == "force_kill_ready":
+            # Force kill ready state - button is "Force Kill" and enabled
+            self.buttons["run_plate"].setEnabled(True)
+            self.buttons["run_plate"].setText("Force Kill")
+        elif is_running:
+            # Running state - button is "Stop" and enabled
             self.buttons["run_plate"].setEnabled(True)
             self.buttons["run_plate"].setText("Stop")
         else:
+            # Idle state - button is "Run" and enabled if plates are compiled
             self.buttons["run_plate"].setEnabled(has_compiled)
             self.buttons["run_plate"].setText("Run")
     
     def is_any_plate_running(self) -> bool:
         """
         Check if any plate is currently running.
-        
+
         Returns:
             True if any plate is running, False otherwise
         """
-        return self.execution_state == "running"
+        # Consider "running", "stopping", and "force_kill_ready" states as "busy"
+        return self.execution_state in ("running", "stopping", "force_kill_ready")
     
     def update_status(self, message: str):
         """
@@ -1822,6 +1820,15 @@ class PlateManagerWidget(QWidget):
         self.pipeline_editor = pipeline_editor
         logger.debug("Pipeline editor reference set in plate manager")
 
+    def _find_main_window(self):
+        """Find the main window by traversing parent hierarchy."""
+        widget = self
+        while widget:
+            if hasattr(widget, 'floating_windows'):
+                return widget
+            widget = widget.parent()
+        return None
+
     async def _start_monitoring(self):
         """Start monitoring subprocess execution."""
         if not self.current_process:
@@ -1864,18 +1871,22 @@ class PlateManagerWidget(QWidget):
             await asyncio.sleep(1)  # Check every second
 
     def _on_progress_started(self, max_value: int):
-        """Handle progress started signal (main thread)."""
-        self.progress_bar.setVisible(True)
-        self.progress_bar.setMaximum(max_value)
-        self.progress_bar.setValue(0)
+        """Handle progress started signal - route to status bar."""
+        # Progress is now displayed in the status bar instead of a separate widget
+        # This method is kept for signal compatibility but doesn't need to do anything
+        pass
 
     def _on_progress_updated(self, value: int):
-        """Handle progress updated signal (main thread)."""
-        self.progress_bar.setValue(value)
+        """Handle progress updated signal - route to status bar."""
+        # Progress is now displayed in the status bar instead of a separate widget
+        # This method is kept for signal compatibility but doesn't need to do anything
+        pass
 
     def _on_progress_finished(self):
-        """Handle progress finished signal (main thread)."""
-        self.progress_bar.setVisible(False)
+        """Handle progress finished signal - route to status bar."""
+        # Progress is now displayed in the status bar instead of a separate widget
+        # This method is kept for signal compatibility but doesn't need to do anything
+        pass
 
     def _handle_compilation_error(self, plate_name: str, error_message: str):
         """Handle compilation error on main thread (slot)."""
