@@ -340,8 +340,10 @@ class ZMQServerManagerWidget(QWidget):
         """
         import zmq
         import pickle
+        from openhcs.constants.constants import CONTROL_PORT_OFFSET
+        from openhcs.runtime.zmq_base import get_zmq_transport_url, get_default_transport_mode
 
-        control_port = port + 1000
+        control_port = port + CONTROL_PORT_OFFSET
         control_context = None
         control_socket = None
 
@@ -350,7 +352,11 @@ class ZMQServerManagerWidget(QWidget):
             control_socket = control_context.socket(zmq.REQ)
             control_socket.setsockopt(zmq.LINGER, 0)
             control_socket.setsockopt(zmq.RCVTIMEO, 300)  # 300ms timeout for fast scanning
-            control_socket.connect(f"tcp://localhost:{control_port}")
+
+            # Use transport mode-aware URL (IPC or TCP)
+            transport_mode = get_default_transport_mode()
+            control_url = get_zmq_transport_url(control_port, transport_mode, 'localhost')
+            control_socket.connect(control_url)
 
             # Send ping
             ping_message = {'type': 'ping'}
@@ -542,6 +548,15 @@ class ZMQServerManagerWidget(QWidget):
                         # All images processed
                         info_text = f"✅ Processed {total} images"
 
+                # If no processing info, show memory usage
+                if not info_text:
+                    mem_mb = server.get('memory_mb')
+                    cpu_percent = server.get('cpu_percent')
+                    if mem_mb is not None:
+                        info_text = f"Mem: {mem_mb:.0f}MB"
+                        if cpu_percent is not None:
+                            info_text += f" | CPU: {cpu_percent:.1f}%"
+
                 item = QTreeWidgetItem([display_text, status_text, info_text])
                 item.setData(0, Qt.ItemDataRole.UserRole, server)
                 self.server_tree.addTopLevelItem(item)
@@ -673,37 +688,35 @@ class ZMQServerManagerWidget(QWidget):
         import threading
 
         def kill_servers():
-            from openhcs.runtime.zmq_base import ZMQServer
+            from openhcs.runtime.zmq_base import ZMQClient
             from openhcs.runtime.queue_tracker import GlobalQueueTrackerRegistry
-            failed_ports = []
             registry = GlobalQueueTrackerRegistry()
 
             for port in ports_to_kill:
                 try:
-                    logger.info(f"🔥 FORCE KILL: Killing processes on port {port} (nuclear option - no ZMQ, direct process kill)")
-                    # Force kill: Skip ZMQ entirely, go straight to process killing
-                    # This is the nuclear option - it ALWAYS works
-                    killed = ZMQServer.kill_processes_on_port(port)
-                    # Also try port+1000 (data port for execution servers)
-                    killed += ZMQServer.kill_processes_on_port(port + 1000)
+                    logger.info(f"🔥 FORCE KILL: Force killing server on port {port} (kills workers AND server)")
+                    # Use kill_server_on_port with graceful=False
+                    # This handles both IPC and TCP modes correctly
+                    success = ZMQClient.kill_server_on_port(port, graceful=False)
 
-                    if killed > 0:
-                        logger.info(f"✅ Force killed {killed} process(es) on port {port}")
-                        # Clear queue tracker for this viewer
-                        registry.remove_tracker(port)
-                        self.server_killed.emit(port)
+                    if success:
+                        logger.info(f"✅ Successfully force killed server on port {port}")
                     else:
-                        failed_ports.append(port)
-                        logger.warning(f"❌ No processes found on port {port}")
-                except Exception as e:
-                    failed_ports.append(port)
-                    logger.error(f"❌ Error force killing server on port {port}: {e}")
+                        logger.warning(f"⚠️ Force kill returned False for port {port}, but continuing cleanup")
 
-            # Emit completion signal
-            if failed_ports:
-                self._kill_complete.emit(False, f"Failed to force kill servers on ports: {failed_ports}")
-            else:
-                self._kill_complete.emit(True, "All servers force killed successfully")
+                    # Clear queue tracker for this viewer (always, regardless of kill result)
+                    registry.remove_tracker(port)
+                    self.server_killed.emit(port)
+
+                except Exception as e:
+                    logger.error(f"❌ Error force killing server on port {port}: {e}")
+                    # Still emit signal to trigger refresh and cleanup, even on error
+                    registry.remove_tracker(port)
+                    self.server_killed.emit(port)
+
+            # Always emit success - we've done our best to kill the processes
+            # The refresh will remove any dead entries from the list
+            self._kill_complete.emit(True, "Force kill operation completed (list will refresh)")
 
         thread = threading.Thread(target=kill_servers, daemon=True)
         thread.start()
