@@ -26,8 +26,11 @@ class OpenHCSExecutionServer:
         omero_port: int = 4064,
         omero_user: str = 'root',
         omero_password: str = 'omero-root-password',
-        server_port: int = 7777
+        server_port: int = None
     ):
+        from openhcs.constants.constants import DEFAULT_EXECUTION_SERVER_PORT
+        if server_port is None:
+            server_port = DEFAULT_EXECUTION_SERVER_PORT
         self.omero_data_dir = Path(omero_data_dir) if omero_data_dir else None
         self.omero_host = omero_host
         self.omero_port = omero_port
@@ -274,6 +277,13 @@ class OpenHCSExecutionServer:
 
             logger.info(f"[{execution_id}] Starting execution for plate {plate_id}")
 
+            # Initialize function registry BEFORE executing pipeline code
+            # (pipeline code may import virtual modules like openhcs.cucim)
+            import openhcs.processing.func_registry as func_registry_module
+            with func_registry_module._registry_lock:
+                if not func_registry_module._registry_initialized:
+                    func_registry_module._auto_initialize_registry()
+
             # Reconstruct pipeline by executing the exact generated Python code (same as UI)
             # Use an empty namespace so imports resolve naturally to module-level symbols
             namespace: Dict[str, Any] = {}
@@ -378,17 +388,12 @@ class OpenHCSExecutionServer:
             # Execute using standard compile→execute phases
             logger.info(f"[{execution_id}] Executing pipeline...")
 
-            # Determine wells to process
-            if config_params and config_params.get('well_filter'):
-                wells = config_params['well_filter']
-            elif hasattr(global_config, 'step_well_filter_config') and global_config.step_well_filter_config and global_config.step_well_filter_config.well_filter:
-                wells = global_config.step_well_filter_config.well_filter
-            else:
-                wells = orchestrator.get_component_keys(MULTIPROCESSING_AXIS)
-
+            # CRITICAL: Compiler now resolves well_filter_config from pipeline_config automatically
+            # No need to extract it manually here - just pass None to compile_pipelines
+            # The compiler will check orchestrator.pipeline_config.well_filter_config and use it
             compilation = orchestrator.compile_pipelines(
                 pipeline_definition=pipeline_steps,
-                well_filter=wells,
+                well_filter=None,
             )
             compiled_contexts = compilation['compiled_contexts']
 
@@ -430,8 +435,8 @@ class OpenHCSExecutionServer:
             if hasattr(step, 'napari_streaming_config') and step.napari_streaming_config:
                 new_config = replace(
                     step.napari_streaming_config,
-                    napari_host=client_host,
-                    napari_port=client_port
+                    host=client_host,
+                    port=client_port
                 )
                 step = replace(step, napari_streaming_config=new_config)
             updated_steps.append(step)
@@ -457,6 +462,7 @@ class OpenHCSExecutionServer:
 def main():
     """Entry point for execution server."""
     import argparse
+    from openhcs.constants.constants import DEFAULT_EXECUTION_SERVER_PORT
 
     parser = argparse.ArgumentParser(description='OpenHCS Execution Server')
     parser.add_argument('--omero-data-dir', type=Path, default=None,
@@ -465,7 +471,7 @@ def main():
     parser.add_argument('--omero-port', type=int, default=4064)
     parser.add_argument('--omero-user', default='root')
     parser.add_argument('--omero-password', default='omero-root-password')
-    parser.add_argument('--port', type=int, default=7777)
+    parser.add_argument('--port', type=int, default=DEFAULT_EXECUTION_SERVER_PORT)
     parser.add_argument('--log-file', type=Path)
     parser.add_argument('--log-level', default='INFO', choices=['DEBUG', 'INFO', 'WARNING', 'ERROR'])
 
