@@ -9,8 +9,13 @@ import threading
 import os
 from abc import ABC, abstractmethod
 from pathlib import Path
+from typing import Optional
 import pickle
 from openhcs.runtime.zmq_messages import ControlMessageType, ResponseType, MessageFields, PongResponse, SocketType, ImageAck
+from openhcs.constants.constants import (
+    CONTROL_PORT_OFFSET, IPC_SOCKET_DIR_NAME, IPC_SOCKET_PREFIX, IPC_SOCKET_EXTENSION
+)
+from openhcs.core.config import TransportMode
 
 logger = logging.getLogger(__name__)
 
@@ -20,28 +25,130 @@ logger = logging.getLogger(__name__)
 SHARED_ACK_PORT = 7555
 
 
+def get_default_transport_mode() -> TransportMode:
+    """
+    Get the default transport mode for the current platform.
+
+    Windows doesn't support IPC (POSIX named pipes), so use TCP with localhost.
+    Unix-like systems (Linux/macOS) use IPC for better performance.
+
+    Returns:
+        TransportMode.TCP on Windows, TransportMode.IPC on Unix/macOS
+    """
+    return TransportMode.TCP if platform.system() == 'Windows' else TransportMode.IPC
+
+
+# ============================================================================
+# ZMQ Transport Utilities
+# ============================================================================
+
+def _get_ipc_socket_path(port: int) -> Optional[Path]:
+    """Get IPC socket path for a given port (Unix/Mac only).
+
+    Args:
+        port: Port number to generate socket path for
+
+    Returns:
+        Path to IPC socket file, or None on Windows
+    """
+    if platform.system() == 'Windows':
+        return None  # Windows uses named pipes, not file paths
+
+    ipc_dir = Path.home() / ".openhcs" / IPC_SOCKET_DIR_NAME
+    socket_name = f"{IPC_SOCKET_PREFIX}-{port}{IPC_SOCKET_EXTENSION}"
+    return ipc_dir / socket_name
+
+
+def _remove_ipc_socket(port: int) -> bool:
+    """Remove stale IPC socket file for a given port.
+
+    Args:
+        port: Port number whose socket should be removed
+
+    Returns:
+        True if socket was removed, False otherwise
+    """
+    socket_path = _get_ipc_socket_path(port)
+    if socket_path and socket_path.exists():
+        try:
+            socket_path.unlink()
+            logger.info(f"🧹 Removed stale IPC socket: {socket_path}")
+            return True
+        except Exception as e:
+            logger.warning(f"Failed to remove stale IPC socket {socket_path}: {e}")
+            return False
+    return False
+
+
+def get_zmq_transport_url(port: int, transport_mode: TransportMode, host: str = 'localhost') -> str:
+    """Generate ZMQ transport URL based on mode and platform.
+
+    Args:
+        port: Port number (used for both IPC and TCP)
+        transport_mode: IPC or TCP
+        host: Host for TCP mode (ignored for IPC)
+
+    Returns:
+        ZMQ transport URL string
+
+    Raises:
+        ValueError: If transport_mode is invalid (fail-loud)
+
+    Examples:
+        >>> get_zmq_transport_url(5555, TransportMode.IPC)  # Windows
+        'ipc://openhcs-zmq-5555'
+
+        >>> get_zmq_transport_url(5555, TransportMode.IPC)  # Unix/Mac
+        'ipc:///home/user/.openhcs/ipc/openhcs-zmq-5555.sock'
+
+        >>> get_zmq_transport_url(5555, TransportMode.TCP, 'localhost')
+        'tcp://localhost:5555'
+    """
+    if transport_mode == TransportMode.IPC:
+        if platform.system() == 'Windows':
+            # Windows named pipes: ipc://openhcs-zmq-{port}
+            return f"ipc://{IPC_SOCKET_PREFIX}-{port}"
+        else:
+            # Unix domain sockets: use helper to get path and ensure directory exists
+            socket_path = _get_ipc_socket_path(port)
+            socket_path.parent.mkdir(parents=True, exist_ok=True)  # Fail-loud if permission denied
+            return f"ipc://{socket_path}"
+    elif transport_mode == TransportMode.TCP:
+        return f"tcp://{host}:{port}"
+    else:
+        # Fail-loud for invalid enum (should never happen with proper typing)
+        raise ValueError(f"Invalid transport_mode: {transport_mode}")
+
+
 # Global ack listener singleton
 _ack_listener_thread = None
 _ack_listener_lock = threading.Lock()
 _ack_listener_running = False
+_ack_listener_transport_mode = None
 
 
-def start_global_ack_listener():
+def start_global_ack_listener(transport_mode: TransportMode = None):
     """Start the global ack listener thread (singleton).
 
     This thread listens on SHARED_ACK_PORT for acknowledgments from all viewers
     and routes them to the appropriate queue trackers.
 
     Safe to call multiple times - only starts once.
+
+    Args:
+        transport_mode: Transport mode to use (IPC or TCP). Defaults to IPC.
     """
-    global _ack_listener_thread, _ack_listener_running
+    global _ack_listener_thread, _ack_listener_running, _ack_listener_transport_mode
 
     with _ack_listener_lock:
         if _ack_listener_running:
             logger.debug("Ack listener already running")
             return
 
-        logger.info(f"Starting global ack listener on port {SHARED_ACK_PORT}")
+        # Set transport mode (default to IPC on Unix, TCP on Windows)
+        _ack_listener_transport_mode = transport_mode or get_default_transport_mode()
+
+        logger.info(f"Starting global ack listener on port {SHARED_ACK_PORT} with {_ack_listener_transport_mode.value} transport")
         _ack_listener_running = True
         _ack_listener_thread = threading.Thread(
             target=_ack_listener_loop,
@@ -65,8 +172,9 @@ def _ack_listener_loop():
 
     try:
         socket = context.socket(zmq.PULL)
-        socket.bind(f"tcp://*:{SHARED_ACK_PORT}")
-        logger.info(f"✅ Ack listener bound to port {SHARED_ACK_PORT}")
+        ack_url = get_zmq_transport_url(SHARED_ACK_PORT, _ack_listener_transport_mode, '*')
+        socket.bind(ack_url)
+        logger.info(f"✅ Ack listener bound to {ack_url}")
 
         while _ack_listener_running:
             try:
@@ -132,13 +240,15 @@ def stop_global_ack_listener():
 class ZMQServer(ABC):
     """ABC for ZMQ servers - dual-channel pattern with ping/pong handshake."""
 
-    def __init__(self, port, host='*', log_file_path=None, data_socket_type=None):
+    def __init__(self, port, host='*', log_file_path=None, data_socket_type=None, transport_mode=None):
         import zmq
         self.port = port
         self.host = host
-        self.control_port = port + 1000
+        self.control_port = port + CONTROL_PORT_OFFSET
         self.log_file_path = log_file_path
         self.data_socket_type = data_socket_type if data_socket_type is not None else zmq.PUB
+        # Windows doesn't support IPC (POSIX named pipes), so use TCP with localhost
+        self.transport_mode = transport_mode or get_default_transport_mode()
         self.zmq_context = None
         self.data_socket = None
         self.control_socket = None
@@ -154,14 +264,26 @@ class ZMQServer(ABC):
             self.zmq_context = zmq.Context()
             self.data_socket = self.zmq_context.socket(self.data_socket_type)
             self.data_socket.setsockopt(zmq.LINGER, 0)
-            self.data_socket.bind(f"tcp://{self.host}:{self.port}")
+
+            # Set high water mark for SUB/PULL sockets to prevent message drops
+            # when viewer is busy processing (e.g., creating shapes layers that take 2-3 seconds)
+            # REP sockets don't need HWM since they're synchronous (one request at a time)
+            if self.data_socket_type in (zmq.SUB, zmq.PULL):
+                self.data_socket.setsockopt(zmq.RCVHWM, 100000)  # Buffer up to 100k messages
+                socket_type_name = "SUB" if self.data_socket_type == zmq.SUB else "PULL"
+                logger.info(f"ZMQ {socket_type_name} socket RCVHWM set to 100000 to prevent drops during blocking operations")
+
+            data_url = get_zmq_transport_url(self.port, self.transport_mode, self.host)
+            control_url = get_zmq_transport_url(self.control_port, self.transport_mode, self.host)
+
+            self.data_socket.bind(data_url)
             if self.data_socket_type == zmq.SUB:
                 self.data_socket.setsockopt(zmq.SUBSCRIBE, b"")
             self.control_socket = self.zmq_context.socket(zmq.REP)
             self.control_socket.setsockopt(zmq.LINGER, 0)
-            self.control_socket.bind(f"tcp://{self.host}:{self.control_port}")
+            self.control_socket.bind(control_url)
             self._running = True
-            logger.info(f"ZMQ Server started on {self.port} ({SocketType.from_zmq_constant(self.data_socket_type).get_display_name()}), control {self.control_port}")
+            logger.info(f"ZMQ Server started on {data_url} ({SocketType.from_zmq_constant(self.data_socket_type).get_display_name()}), control {control_url}")
 
     def stop(self):
         with self._lock:
@@ -229,7 +351,8 @@ class ZMQServer(ABC):
                 for line in result.stdout.split('\n'):
                     if f':{port}' in line and 'LISTENING' in line:
                         try:
-                            subprocess.run(['taskkill', '/F', '/PID', line.split()[-1]], timeout=1)
+                            # Graceful termination (no /F flag) - works without UAC for user processes
+                            subprocess.run(['taskkill', '/PID', line.split()[-1]], timeout=1)
                             killed += 1
                         except:
                             pass
@@ -349,11 +472,13 @@ class ZMQServer(ABC):
 class ZMQClient(ABC):
     """ABC for ZMQ clients - dual-channel pattern with auto-spawning."""
 
-    def __init__(self, port, host='localhost', persistent=True):
+    def __init__(self, port, host='localhost', persistent=True, transport_mode=None):
         self.port = port
         self.host = host
-        self.control_port = port + 1000
+        self.control_port = port + CONTROL_PORT_OFFSET
         self.persistent = persistent
+        # Windows doesn't support IPC (POSIX named pipes), so use TCP with localhost
+        self.transport_mode = transport_mode or get_default_transport_mode()
         self.zmq_context = None
         self.data_socket = None
         self.control_socket = None
@@ -407,9 +532,11 @@ class ZMQClient(ABC):
     def _setup_client_sockets(self):
         import zmq
         self.zmq_context = zmq.Context()
+        data_url = get_zmq_transport_url(self.port, self.transport_mode, self.host)
+
         self.data_socket = self.zmq_context.socket(zmq.SUB)
         self.data_socket.setsockopt(zmq.LINGER, 0)
-        self.data_socket.connect(f"tcp://{self.host}:{self.port}")
+        self.data_socket.connect(data_url)
         self.data_socket.setsockopt(zmq.SUBSCRIBE, b"")
         time.sleep(0.1)
 
@@ -428,11 +555,13 @@ class ZMQClient(ABC):
     def _try_connect_to_existing(self, port):
         import zmq
         try:
+            control_url = get_zmq_transport_url(port + CONTROL_PORT_OFFSET, self.transport_mode, self.host)
+
             ctx = zmq.Context()
             sock = ctx.socket(zmq.REQ)
             sock.setsockopt(zmq.LINGER, 0)
             sock.setsockopt(zmq.RCVTIMEO, 500)
-            sock.connect(f"tcp://{self.host}:{port + 1000}")
+            sock.connect(control_url)
             sock.send(pickle.dumps({'type': 'ping'}))
             response = pickle.loads(sock.recv())
             return response.get('type') == 'pong' and response.get('ready')
@@ -454,6 +583,8 @@ class ZMQClient(ABC):
             time.sleep(0.2)
         else:
             return False
+        control_url = get_zmq_transport_url(self.control_port, self.transport_mode, self.host)
+
         start = time.time()
         while time.time() - start < timeout:
             try:
@@ -461,7 +592,7 @@ class ZMQClient(ABC):
                 sock = ctx.socket(zmq.REQ)
                 sock.setsockopt(zmq.LINGER, 0)
                 sock.setsockopt(zmq.RCVTIMEO, 1000)
-                sock.connect(f"tcp://{self.host}:{self.control_port}")
+                sock.connect(control_url)
                 sock.send(pickle.dumps({'type': 'ping'}))
                 response = pickle.loads(sock.recv())
                 if response.get('type') == 'pong' and response.get('ready'):
@@ -481,17 +612,23 @@ class ZMQClient(ABC):
         return False
 
     def _is_port_in_use(self, port):
-        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        sock.settimeout(0.1)
-        try:
-            sock.bind(('localhost', port))
-            sock.close()
-            return False
-        except OSError:
-            sock.close()
-            return True
-        except:
-            return False
+        if self.transport_mode == TransportMode.IPC:
+            # Use helper function to check IPC socket existence
+            socket_path = _get_ipc_socket_path(port)
+            return socket_path.exists() if socket_path else False
+        else:
+            # TCP mode - check if port is bound
+            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            sock.settimeout(0.1)
+            try:
+                sock.bind(('localhost', port))
+                sock.close()
+                return False
+            except OSError:
+                sock.close()
+                return True
+            except:
+                return False
 
     def _find_free_port(self):
         with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
@@ -500,6 +637,12 @@ class ZMQClient(ABC):
 
     def _kill_processes_on_port(self, port):
         try:
+            # For IPC mode, remove stale socket files using helper function
+            if self.transport_mode == TransportMode.IPC:
+                _remove_ipc_socket(port)
+                return  # IPC mode doesn't use TCP ports, so skip process killing
+
+            # TCP mode - kill processes using the port
             system = platform.system()
             if system in ["Linux", "Darwin"]:
                 result = subprocess.run(['lsof', '-ti', f'TCP:{port}', '-sTCP:LISTEN'], capture_output=True, text=True, timeout=2)
@@ -514,28 +657,34 @@ class ZMQClient(ABC):
                 for line in result.stdout.split('\n'):
                     if f':{port}' in line and 'LISTENING' in line:
                         try:
-                            subprocess.run(['taskkill', '/F', '/PID', line.split()[-1]], timeout=1)
+                            # Graceful termination (no /F flag) - works without UAC for user processes
+                            subprocess.run(['taskkill', '/PID', line.split()[-1]], timeout=1)
                         except:
                             pass
         except:
             pass
 
     @staticmethod
-    def scan_servers(ports, host='localhost', timeout_ms=200):
+    def scan_servers(ports, host='localhost', timeout_ms=200, transport_mode=None):
         import zmq
+        # Windows doesn't support IPC, so use TCP with localhost
+        transport_mode = transport_mode or get_default_transport_mode()
         servers = []
         for port in ports:
             try:
+                control_port = port + CONTROL_PORT_OFFSET
+                control_url = get_zmq_transport_url(control_port, transport_mode, host)
+
                 ctx = zmq.Context()
                 sock = ctx.socket(zmq.REQ)
                 sock.setsockopt(zmq.LINGER, 0)
                 sock.setsockopt(zmq.RCVTIMEO, timeout_ms)
-                sock.connect(f"tcp://{host}:{port + 1000}")
+                sock.connect(control_url)
                 sock.send(pickle.dumps({'type': 'ping'}))
                 pong = pickle.loads(sock.recv())
                 if pong.get('type') == 'pong':
                     pong['port'] = port
-                    pong['control_port'] = port + 1000
+                    pong['control_port'] = control_port
                     servers.append(pong)
             except:
                 pass
@@ -548,7 +697,7 @@ class ZMQClient(ABC):
         return servers
 
     @staticmethod
-    def kill_server_on_port(port, graceful=True, timeout=5.0):
+    def kill_server_on_port(port, graceful=True, timeout=5.0, transport_mode=None, host='localhost'):
         """
         Kill server on specified port.
 
@@ -557,89 +706,101 @@ class ZMQClient(ABC):
             graceful: If True, kills workers only (server stays alive).
                      If False, kills workers AND server (port becomes free).
             timeout: Timeout in seconds for server response
+            transport_mode: TransportMode (IPC or TCP). If None, uses platform default.
+            host: Host for TCP mode (ignored for IPC)
 
         Returns:
             bool: True if operation succeeded
         """
         import zmq
+        transport_mode = transport_mode or get_default_transport_mode()
         msg_type = 'shutdown' if graceful else 'force_shutdown'
         shutdown_sent = False
         shutdown_acknowledged = False
 
         def is_port_free(port):
-            """Check if port is free (not in use)."""
-            sock_test = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            sock_test.settimeout(0.1)
-            try:
-                sock_test.bind(('localhost', port))
-                sock_test.close()
-                return True
-            except OSError:
-                return False
-            finally:
+            """Check if port is free (not in use) - only for TCP mode."""
+            if transport_mode == TransportMode.IPC:
+                # IPC mode - check if socket file exists
+                socket_path = _get_ipc_socket_path(port)
+                return not (socket_path and socket_path.exists())
+            else:
+                # TCP mode - check if port is bound
+                sock_test = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+                sock_test.settimeout(0.1)
                 try:
+                    sock_test.bind(('localhost', port))
                     sock_test.close()
-                except:
-                    pass
+                    return True
+                except OSError:
+                    return False
+                finally:
+                    try:
+                        sock_test.close()
+                    except:
+                        pass
 
         try:
+            control_port = port + CONTROL_PORT_OFFSET
+            control_url = get_zmq_transport_url(control_port, transport_mode, host)
+
             ctx = zmq.Context()
             sock = ctx.socket(zmq.REQ)
             sock.setsockopt(zmq.LINGER, 0)
-            sock.setsockopt(zmq.RCVTIMEO, int(timeout * 1000))
-            sock.connect(f"tcp://localhost:{port + 1000}")
-            sock.send(pickle.dumps({'type': msg_type}))
-            shutdown_sent = True
-            ack = pickle.loads(sock.recv())
-            if ack.get('type') == 'shutdown_ack':
-                shutdown_acknowledged = True
+            sock.connect(control_url)
 
-                if graceful:
-                    # Graceful shutdown: workers killed, server stays alive
-                    # Port should still be in use
+            if graceful:
+                # Graceful shutdown: wait for ack (server stays alive)
+                sock.setsockopt(zmq.RCVTIMEO, int(timeout * 1000))
+                sock.send(pickle.dumps({'type': msg_type}))
+                shutdown_sent = True
+                ack = pickle.loads(sock.recv())
+                if ack.get('type') == 'shutdown_ack':
                     logger.info(f"✅ Server on port {port} acknowledged graceful shutdown (workers killed, server alive)")
                     return True
-                else:
-                    # Force shutdown: workers AND server killed
-                    # Port should become free
-                    logger.info(f"✅ Server on port {port} acknowledged force shutdown")
-                    # Wait for server to actually close
-                    time.sleep(1.5)
-                    # Verify port is actually closed
-                    if is_port_free(port):
-                        logger.info(f"✅ Port {port} is now free")
-                        return True
-                    # Port still in use, but server acknowledged - give it more time
-                    time.sleep(1.0)
-                    # Check again
-                    if is_port_free(port):
-                        logger.info(f"✅ Port {port} is now free (after extra wait)")
-                        return True
-                    # Server acknowledged but port still in use - consider it success anyway
-                    # (might be in TIME_WAIT state, or server is still cleaning up)
-                    logger.info(f"✅ Server on port {port} acknowledged shutdown but port still in use (may be TIME_WAIT) - considering success")
+            else:
+                # Force shutdown: send message and immediately kill - don't wait for ack
+                # Server is dying, it can't reliably send ack anyway
+                sock.setsockopt(zmq.SNDTIMEO, 1000)  # 1s timeout for send
+                try:
+                    sock.send(pickle.dumps({'type': msg_type}))
+                    logger.info(f"🔥 Sent force shutdown message to port {port}")
+                except Exception as send_error:
+                    logger.warning(f"⚠️ Failed to send force shutdown message: {send_error}")
+
+                # Don't wait for ack - immediately kill the server
+                if transport_mode == TransportMode.IPC:
+                    # IPC mode - remove socket files
+                    _remove_ipc_socket(port)
+                    _remove_ipc_socket(control_port)
+                    logger.info(f"✅ Removed IPC socket files for ports {port} and {control_port}")
                     return True
+                else:
+                    # TCP mode - kill processes on ports
+                    killed = sum(ZMQServer.kill_processes_on_port(p) for p in [port, control_port])
+                    logger.info(f"✅ Force killed {killed} processes on ports {port} and {control_port}")
+                    return killed > 0
+
         except Exception as e:
-            # If we sent the shutdown message but got a timeout on response,
-            # the server might still be shutting down - verify port status
-            if shutdown_sent:
-                if graceful:
-                    # Graceful shutdown timeout - server might have killed workers but didn't respond
-                    # This is OK, consider it success
-                    logger.info(f"⚠️ Graceful shutdown message sent to port {port}, but no response received: {e}")
-                    logger.info(f"✅ Considering graceful shutdown successful (workers likely killed)")
+            # Connection failed - server might not exist or wrong transport mode
+            logger.warning(f"❌ Failed to connect to server on port {port} ({transport_mode.value} mode): {e}")
+
+            if not graceful:
+                # Force shutdown failed via ZMQ, try killing processes directly
+                if transport_mode == TransportMode.IPC:
+                    # IPC mode - remove socket files
+                    _remove_ipc_socket(port)
+                    _remove_ipc_socket(control_port)
+                    logger.info(f"Removed IPC socket files for ports {port} and {control_port}")
                     return True
                 else:
-                    # Force shutdown timeout - check if port is free
-                    logger.info(f"⚠️ Force shutdown message sent to port {port}, but no response received: {e}")
-                    time.sleep(2.0)  # Give server time to shut down
-                    # Check if port is now free
-                    if is_port_free(port):
-                        # Port is free, shutdown was successful despite timeout
-                        logger.info(f"✅ Port {port} is now free (shutdown succeeded despite timeout)")
-                        return True
-                    # Port still in use - shutdown may have failed
-                    logger.warning(f"❌ Port {port} still in use after force shutdown timeout")
+                    # TCP mode - kill processes on ports
+                    killed = sum(ZMQServer.kill_processes_on_port(p) for p in [port, control_port])
+                    logger.info(f"Force killed {killed} processes on ports {port} and {control_port}")
+                    return killed > 0
+
+            logger.warning(f"❌ Failed to shutdown server on port {port} gracefully")
+            return False
         finally:
             try:
                 sock.close()
@@ -647,18 +808,7 @@ class ZMQClient(ABC):
             except:
                 pass
 
-        # If shutdown was acknowledged, consider it a success even if we couldn't verify port closure
-        # (the server confirmed it's shutting down, which is good enough)
-        if shutdown_acknowledged:
-            logger.info(f"✅ Shutdown acknowledged for port {port}, considering it successful")
-            return True
-
-        if not graceful:
-            # Force shutdown failed via ZMQ, try killing processes directly
-            killed = sum(ZMQServer.kill_processes_on_port(p) for p in [port, port + 1000])
-            logger.info(f"Force killed {killed} processes on ports {port} and {port + 1000}")
-            return killed > 0
-
+        # Graceful shutdown failed - no ack received
         logger.warning(f"❌ Failed to shutdown server on port {port} gracefully")
         return False
 
